import * as i0 from '@angular/core';
import { InjectionToken, inject, Injector, TransferState, makeStateKey, Injectable, Directive, Input, makeEnvironmentProviders, APP_INITIALIZER, NgModule } from '@angular/core';
import { HttpContextToken, HttpClient, HttpContext, HttpErrorResponse, HTTP_INTERCEPTORS } from '@angular/common/http';
import { shareReplay, tap, map, distinctUntilChanged, catchError, filter, take, switchMap, finalize } from 'rxjs/operators';
import { firstValueFrom, BehaviorSubject, of, Subscription, first, throwError } from 'rxjs';
import { Storage } from '@bravobit/bb-foundation/storage';
import { HTTP_CONFIG, HttpError } from '@bravobit/bb-foundation/http';
import { Platform } from '@angular/cdk/platform';
import { Router } from '@angular/router';

const AUTH_CONFIG = new InjectionToken('auth config');
const AUTH_REDIRECT_HANDLER = new InjectionToken('auth redirect handler');

const USE_AUTHORIZATION = new HttpContextToken(() => true);

class JwtHelper {
    decode(token) {
        try {
            if (token === null || token === undefined) {
                return null;
            }
            const json = JSON.parse(this.urlDecode(token.split('.')[1]));
            return this.parse(json);
        }
        catch {
            return null;
        }
    }
    urlDecode(token) {
        const value = token || '';
        let output = value
            .replace(/-/g, '+')
            .replace(/_/g, '/');
        switch (output.length % 4) {
            case 0:
                break;
            case 2:
                output += '==';
                break;
            case 3:
                output += '=';
                break;
            default:
                throw 'Illegal base64url string!';
        }
        try {
            return this.baseDecodeUnicode(output);
        }
        catch {
            return atob(output);
        }
    }
    baseDecodeUnicode = (value) => {
        return decodeURIComponent(atob(value).replace(/(.)/g, (_, p) => {
            let code = p.charCodeAt(0).toString(16).toUpperCase();
            if (code.length < 2) {
                code = '0' + code;
            }
            return '%' + code;
        }));
    };
    parse = (data) => {
        return {
            id: data['jti'] ?? null,
            type: data['typ'] ?? null,
            audience: data['aud'] ?? null,
            issuer: data['iss'] ?? null,
            subject: data['sub'] ?? null,
            role: data['role'] ?? null,
            notValidBefore: this.parseDate(data['nbf']),
            expiresAt: this.parseDate(data['exp']),
            issuedAt: this.parseDate(data['iat'])
        };
    };
    parseDate = (epochInSeconds) => {
        if (!epochInSeconds || epochInSeconds <= 0) {
            return null;
        }
        return new Date(epochInSeconds * 1000);
    };
}

class AuthVerifyProvider {
    _code;
    _verifyToken;
    _endpoint;
    constructor(_code, _verifyToken, _endpoint) {
        this._code = _code;
        this._verifyToken = _verifyToken;
        this._endpoint = _endpoint;
    }
    async authenticate(httpClient) {
        // Execute API call.
        const data$ = httpClient.post(this._endpoint, {
            token: this._code,
            verify_token: this._verifyToken
        });
        const data = await firstValueFrom(data$);
        return {
            accessToken: data?.token,
            refreshToken: data?.refresh_token,
            user: data?.user
        };
    }
}

class AuthEmailProvider {
    _email;
    _password;
    _endpoint;
    constructor(_email, _password, _endpoint) {
        this._email = _email;
        this._password = _password;
        this._endpoint = _endpoint;
    }
    async authenticate(httpClient) {
        // Execute API call.
        const data$ = httpClient.post(this._endpoint, {
            email: this._email,
            password: this._password
        });
        const data = await firstValueFrom(data$);
        return {
            accessToken: data?.token,
            refreshToken: data?.refresh_token,
            user: data?.user,
            provider: data?.provider,
            verifyToken: data?.verify_token
        };
    }
}

class AuthSession {
    // Readonly data.
    _jwt = new JwtHelper();
    _storage;
    _accessTokenStorageKey;
    _refreshTokenStorageKey;
    _userStorageKey;
    // Token strings.
    _accessTokenString = null;
    _refreshTokenString = null;
    // Token payloads.
    _accessTokenPayload = null;
    _refreshTokenPayload = null;
    // Private user data.
    _user$ = new BehaviorSubject(null);
    // Public user data.
    user = this._user$.pipe(shareReplay({ refCount: true, bufferSize: 1 }));
    constructor(options) {
        const applicationId = options?.id ?? 'ng';
        // Setting up the readonly storage keys.
        this._accessTokenStorageKey = this.generateKey(applicationId, 'au_act');
        this._refreshTokenStorageKey = this.generateKey(applicationId, 'au_rft');
        this._userStorageKey = this.generateKey(applicationId, 'au_usr');
        // Setting up the storage.
        this._storage = options?.storage ?? null;
        // Init methods.
        this.restoreFromStorage();
    }
    get snapshot() {
        return {
            user: this._user$.getValue(),
            accessToken: this.accessToken,
            refreshToken: this.refreshToken
        };
    }
    get accessToken() {
        return this.isTokenValid(this._accessTokenPayload)
            ? this._accessTokenString
            : null;
    }
    get refreshToken() {
        return this.isTokenValid(this.refreshTokenPayload)
            ? this._refreshTokenString
            : null;
    }
    get accessTokenPayload() {
        return this._accessTokenPayload ?? null;
    }
    get refreshTokenPayload() {
        return this._refreshTokenPayload ?? null;
    }
    authenticated() {
        return this.isTokenValid(this._accessTokenPayload) || this.isTokenValid(this._refreshTokenPayload);
    }
    setTokens(accessToken, refreshToken) {
        this.setAccessToken(accessToken);
        this.setRefreshToken(refreshToken);
        this.syncTokensInStorage();
    }
    setUser(user) {
        this._user$.next(user ?? null);
        this.syncUserInStorage();
    }
    clear() {
        this.setTokens(null, null);
        this.setUser(null);
    }
    restoreFromStorage() {
        if (!this._storage) {
            return;
        }
        // Set the access token.
        const accessToken = this._storage.get(this._accessTokenStorageKey);
        this.setAccessToken(accessToken);
        // Set the refresh token.
        const refreshToken = this._storage.get(this._refreshTokenStorageKey);
        this.setRefreshToken(refreshToken);
        // Set the user if we have any correct token payloads.
        if (this._accessTokenPayload || this._refreshTokenPayload) {
            const user = this._storage.get(this._userStorageKey);
            this._user$.next(user ?? null); // Note: just settings here instead of setUser() because of syncing to the storage.
        }
    }
    syncTokensInStorage() {
        if (!this._storage) {
            return;
        }
        // Set the access token if completely valid.
        if (!!this.accessToken) {
            this._storage.set(this._accessTokenStorageKey, this._accessTokenString, {
                expires: this._accessTokenPayload?.expiresAt
            });
        }
        else {
            this._storage.remove(this._accessTokenStorageKey);
        }
        // Set the refresh token if completely valid.
        if (!!this.refreshToken) {
            this._storage.set(this._refreshTokenStorageKey, this._refreshTokenString, {
                expires: this._refreshTokenPayload?.expiresAt
            });
        }
        else {
            this._storage.remove(this._refreshTokenStorageKey);
        }
    }
    syncUserInStorage() {
        if (!this._storage) {
            return;
        }
        const user = this._user$.getValue();
        if (!user) {
            return this._storage.remove(this._userStorageKey);
        }
        const date = new Date();
        date.setFullYear(date.getFullYear() + 1);
        this._storage.set(this._userStorageKey, user, {
            expires: new Date(date.getTime())
        });
    }
    setAccessToken(value) {
        this._accessTokenString = value ?? null;
        this._accessTokenPayload = this._jwt.decode(this._accessTokenString);
    }
    setRefreshToken(value) {
        this._refreshTokenString = value ?? null;
        this._refreshTokenPayload = this._jwt.decode(this._refreshTokenString);
    }
    generateKey = (applicationId, key) => {
        return [applicationId, key].join('_');
    };
    isTokenValid = (token) => {
        if (!token) {
            return false;
        }
        return token?.expiresAt?.getTime() > Date.now();
    };
}

class Auth {
    // Dependencies.
    _storage = inject(Storage);
    _injector = inject(Injector);
    _platform = inject(Platform);
    _httpClient = inject(HttpClient);
    _state = inject(TransferState, { optional: true });
    _config = inject(AUTH_CONFIG, { optional: true });
    _httpConfig = inject(HTTP_CONFIG, { optional: true });
    _handler = inject(AUTH_REDIRECT_HANDLER, { optional: true });
    // Readonly data.
    _authStateKey = makeStateKey(`bbAuthStateKey`);
    _httpAlias = this._httpConfig?.defaultAlias ?? null;
    session;
    user;
    _refreshHandler = null;
    constructor() {
        // We select a storage strategy based on the server/browser.
        // Only cookies CAN work on the server.
        const storageStrategy = this._platform.isBrowser
            ? this._storage.select(["cookie" /* StorageOption.Cookie */, "local" /* StorageOption.Local */])
            : this._storage.cookie;
        // Starting the new session.
        this.session = new AuthSession({
            id: this._config?.applicationId,
            storage: storageStrategy
        });
        this.user = this.session.user;
    }
    initialize() {
        return async () => {
            // Check if the app should bootstrap the authentication.
            const shouldBootstrap = this._config?.bootstrap ?? true;
            if (!shouldBootstrap) {
                return this.handleAutoRefreshing();
            }
            // Only retrieve from the server when we are actually authenticated.
            if (!this.session.authenticated()) {
                return;
            }
            // Get the key from the server state.
            if (this._state && this._state?.hasKey(this._authStateKey)) {
                const user = this._state?.get(this._authStateKey, null) ?? null;
                return this.session.setUser(user);
            }
            // Try to fetch the user from the server.
            const user$ = this.me();
            const user = await firstValueFrom(user$, { defaultValue: null });
            // Set the state if exists.
            if (this._state) {
                this._state?.set(this._authStateKey, user ?? null);
            }
            // Save the user in the storage and handle auto refreshing.
            this.session.setUser(user);
            this.handleAutoRefreshing();
        };
    }
    me() {
        const url = this.getUrl('auth/me');
        return this._httpClient.get(url);
    }
    async signIn(provider, as) {
        const { accessToken, refreshToken, user, ...result } = await provider.authenticate(this._httpClient);
        // Check if the role matches.
        const role = user?.role ?? null;
        if (as && !as.includes(role)) {
            throw new Error('Invalid role.');
        }
        // Validate if the provider is one of the available
        // providers then return the user object and the provider.
        const apiProvider = result?.provider ?? null;
        const apiVerifyToken = result?.verifyToken ?? null;
        const availableProviders = this._config?.providers ?? ['email', 'sms', 'totp'];
        if (availableProviders.includes(apiProvider)) {
            return { user, provider: apiProvider, verifyToken: apiVerifyToken };
        }
        // Set the tokens in storage.
        this.setTokens(accessToken, refreshToken);
        // Set the user in storage.
        this.session.setUser(user);
        // Return the user.
        return { user };
    }
    async signInWithEmail(email, password, as) {
        const url = this.getUrl('auth/login');
        return this.signIn(new AuthEmailProvider(email, password, url), as);
    }
    async signInWithVerifyCode(code, verifyToken) {
        const url = this.getUrl('auth/verify');
        return this.signIn(new AuthVerifyProvider(code, verifyToken, url));
    }
    async resendVerifyCode(verifyToken) {
        const url = this.getUrl('auth/resend');
        const result$ = this._httpClient.post(url, {
            verify_token: verifyToken
        });
        return firstValueFrom(result$);
    }
    async register(data, options) {
        // Execute API call.
        const url = this.getUrl('auth/register');
        const result$ = this._httpClient.post(url, data, options);
        const result = await firstValueFrom(result$);
        // Set the tokens in storage.
        this.setTokens(result?.token, result?.refresh_token);
        // Set the user in storage.
        const user = result?.user;
        this.session.setUser(user);
        // Return the user.
        return user;
    }
    logout() {
        // If we don't have a refresh token just clear the session.
        // Note: We do this because else we try to invalidate
        // an "undefined" refresh token.
        const refreshToken = this.session.refreshToken;
        if (!refreshToken) {
            return this.session.clear();
        }
        // We do have a refresh token, so try to
        // invalidate it in the backend.
        try {
            const url = this.getUrl('auth/logout');
            const headerName = this._config?.http?.header ?? 'Authorization';
            const observable$ = this._httpClient.get(url, {
                headers: { [headerName]: refreshToken }
            });
            firstValueFrom(observable$).then(_ => _).catch(_ => _);
        }
        catch {
            // Do nothing because the tokens will be deleted anyways from the session.
        }
        // Delete the tokens from the session.
        return this.session.clear();
    }
    refresh() {
        // If the refresh token does
        // not exist just return an observable of null.
        const refreshToken = this.session.refreshToken;
        if (!refreshToken) {
            return of(null);
        }
        // Perform the refresh call.
        const headerName = this._config?.http?.header ?? 'Authorization';
        const scheme = this._config?.http?.scheme ?? 'Bearer';
        const url = this.getUrl('auth/refresh');
        const context = new HttpContext().set(USE_AUTHORIZATION, false);
        return this._httpClient.get(url, {
            headers: { [headerName]: `${scheme} ${refreshToken}` },
            context: context
        }).pipe(tap(({ token, refresh_token }) => this.setTokens(token, refresh_token)), map(({ token }) => token));
    }
    async requestPassword(email, extraParams = {}) {
        const url = this.getUrl('auth/reset');
        const observable$ = this._httpClient.post(url, { ...extraParams, email });
        return firstValueFrom(observable$);
    }
    async resetPassword(token, newPassword, extraParams = {}) {
        const url = this.getUrl('auth/reset-password');
        const observable$ = this._httpClient.post(url, { ...extraParams, token, password: newPassword });
        return firstValueFrom(observable$);
    }
    clearAndRedirect() {
        // 1. Delete the tokens from the session.
        this.session.clear();
        if (this._handler) {
            const urlTree = this._handler.onFailedAuthenticated();
            if (typeof urlTree === 'boolean' || urlTree === null || urlTree === undefined) {
                return;
            }
            this.router.navigateByUrl(urlTree).then(_ => _);
            return;
        }
        // 2. Compose the route url.
        const redirectUrl = this._config?.redirects.unauthenticated ?? null;
        // 3. Route back if the user provided a redirect url.
        if (this.router && redirectUrl) {
            const commands = Array.isArray(redirectUrl) ? redirectUrl : [redirectUrl];
            this.router.navigate(commands).then(_ => _);
        }
    }
    setTokens(accessToken, refreshToken) {
        // Set the tokens in our session.
        this.session.setTokens(accessToken, refreshToken);
        // We need to update the auto refresh of the refresh token.
        this.handleAutoRefreshing();
    }
    handleAutoRefreshing() {
        const shouldAutoRefresh = this._config?.autoRefresh ?? false;
        if (!shouldAutoRefresh) {
            return;
        }
        const expiresAt = this.session.refreshTokenPayload?.expiresAt ?? null;
        if (expiresAt === null || !this._platform.isBrowser) {
            return;
        }
        const differenceInMilliseconds = expiresAt.getTime() - Date.now();
        const offsetInMilliseconds = 10_000; // 10 seconds.
        // We want to start the refresh 10 seconds before it expires.
        const actualTiming = differenceInMilliseconds - offsetInMilliseconds;
        if (actualTiming <= 0) {
            return;
        }
        // We need to cap the timings because if
        // we get large numbers it might cause unwanted results.
        const maxTiming = 1000 * 60 * 60 * 24; // 24 hours.
        const cappedTiming = Math.max(1, Math.min(actualTiming, maxTiming));
        try {
            if (this._refreshHandler !== null) {
                clearTimeout?.(this._refreshHandler);
                this._refreshHandler = null;
            }
            this._refreshHandler = window?.setTimeout?.(() => this.autoRefresh(), cappedTiming);
        }
        catch {
            // Just ignore it.
        }
    }
    async autoRefresh() {
        try {
            // We just need to wait for it to refresh.
            const refresh$ = this.refresh();
            await firstValueFrom(refresh$);
        }
        catch {
            // Something went wrong refreshing, we need to clear.
            this.clearAndRedirect();
        }
    }
    get router() {
        return this._injector.get(Router);
    }
    getUrl(endpoint) {
        return [this._httpAlias, endpoint]
            .filter(item => !!item)
            .join('/');
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: Auth, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: Auth, providedIn: 'root' });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: Auth, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [] });

class BbAuthenticated {
    _auth;
    _templateRef;
    _viewContainerRef;
    // Data.
    _valid = false;
    // Templates.
    _elseTemplateRef = null;
    // View refs.
    _thenViewRef = null;
    _elseViewRef = null;
    // Subscriptions.
    _subscription = new Subscription();
    set bbAuthenticatedElse(templateRef) {
        this.assertTemplate('bbAuthenticatedElse', templateRef);
        this._elseTemplateRef = templateRef;
        this.updateView();
    }
    constructor(_auth, _templateRef, _viewContainerRef) {
        this._auth = _auth;
        this._templateRef = _templateRef;
        this._viewContainerRef = _viewContainerRef;
    }
    ngOnInit() {
        const subscription = this._auth.user.pipe(map(user => !!user), distinctUntilChanged()).subscribe(valid => {
            this._valid = valid;
            this.updateView();
        });
        this._subscription.add(subscription);
    }
    ngOnDestroy() {
        this._subscription?.unsubscribe();
    }
    updateView() {
        if (this._valid) {
            if (!this._thenViewRef) {
                this._viewContainerRef.clear();
                this._elseViewRef = null;
                if (this._templateRef) {
                    this._thenViewRef = this._viewContainerRef.createEmbeddedView(this._templateRef);
                }
            }
        }
        else {
            if (!this._elseViewRef) {
                this._viewContainerRef.clear();
                this._thenViewRef = null;
                if (this._elseTemplateRef) {
                    this._elseViewRef = this._viewContainerRef.createEmbeddedView(this._elseTemplateRef);
                }
            }
        }
    }
    assertTemplate(property, templateRef) {
        const isTemplateRefOrNull = !!(!templateRef || templateRef.createEmbeddedView);
        if (!isTemplateRefOrNull) {
            throw new Error(`${property} must be a TemplateRef.`);
        }
    }
    static ngAcceptInputType_bbAuthenticatedElse;
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbAuthenticated, deps: [{ token: Auth }, { token: i0.TemplateRef }, { token: i0.ViewContainerRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.8", type: BbAuthenticated, isStandalone: true, selector: "[bbAuthenticated]", inputs: { bbAuthenticatedElse: "bbAuthenticatedElse" }, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbAuthenticated, decorators: [{
            type: Directive,
            args: [{
                    selector: '[bbAuthenticated]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: Auth }, { type: i0.TemplateRef }, { type: i0.ViewContainerRef }], propDecorators: { bbAuthenticatedElse: [{
                type: Input
            }] } });

const bbAnonymousGuard = (route, state) => {
    const auth = inject(Auth);
    const router = inject(Router);
    const config = inject(AUTH_CONFIG, { optional: true });
    const redirectHandler = inject(AUTH_REDIRECT_HANDLER, { optional: true });
    return auth.user.pipe(map(user => !!user), map(authenticated => {
        if (!authenticated) {
            return true;
        }
        if (redirectHandler) {
            return redirectHandler.onFailedAnonymous(route, state);
        }
        // If we don't have a URL to go to we can just say
        // the user is not allowed in this route by returning false.
        const nextUrl = config?.redirects?.authenticated ?? null;
        if (!nextUrl) {
            return false;
        }
        const commands = Array.isArray(nextUrl) ? nextUrl : [nextUrl];
        return router.createUrlTree(commands);
    }), first());
};

const bbAuthenticatedGuard = (route, state) => {
    const auth = inject(Auth);
    const router = inject(Router);
    const config = inject(AUTH_CONFIG, { optional: true });
    const redirectHandler = inject(AUTH_REDIRECT_HANDLER, { optional: true });
    return auth.user.pipe(map(user => !!user), map(authenticated => {
        if (authenticated) {
            return true;
        }
        if (redirectHandler) {
            return redirectHandler.onFailedAuthenticated(route, state);
        }
        // If we don't have a URL to go to we can just say
        // the user is not allowed in this route by returning false.
        const nextUrl = config?.redirects?.unauthenticated ?? null;
        if (!nextUrl) {
            return false;
        }
        const setRedirectOnFailedAuth = config?.setRedirectOnFailedAuth ?? true;
        const redirectUrl = state?.url ?? null;
        const queryParams = setRedirectOnFailedAuth && redirectUrl ? { redirectUrl } : {};
        const commands = Array.isArray(nextUrl) ? nextUrl : [nextUrl];
        return router.createUrlTree(commands, { queryParams });
    }), first());
};

class AuthInterceptor {
    // Dependencies.
    _auth = inject(Auth);
    _config = inject(AUTH_CONFIG, { optional: true });
    // Readonly data.
    _authHeaderString = this._config?.http?.header ?? 'Authorization';
    _authScheme = this._config?.http?.scheme ?? 'Bearer';
    // Data.
    isRefreshing = false;
    refreshingAccessToken$ = new BehaviorSubject(null);
    intercept(request, next) {
        // 1. Check if the user wants to use the authorization for this request.
        if (!request.context.get(USE_AUTHORIZATION)) {
            return next.handle(request);
        }
        // 2. Compose the new request.
        const accessToken = this.getAccessToken(request);
        const newRequest = this.addAuthorizationHeader(request, accessToken);
        // 3. Handle all errors.
        return next.handle(newRequest).pipe(catchError(error => {
            // Handle the HTTP401 error.
            if ((error instanceof HttpErrorResponse || error instanceof HttpError) && error?.status === 401) {
                return this.handle401Error(request, next);
            }
            // Just re-throw the parsed error.
            return throwError(() => error);
        }));
    }
    handle401Error(request, next) {
        // If already refreshing wait for the refresh token to complete.
        if (this.isRefreshing) {
            return this.refreshingAccessToken$.pipe(filter(accessToken => accessToken !== null), take(1), switchMap(accessToken => next.handle(this.addAuthorizationHeader(request, accessToken))));
        }
        // Set the refreshing to true.
        this.isRefreshing = true;
        this.refreshingAccessToken$.next(null);
        return this._auth.refresh().pipe(switchMap(newAccessToken => {
            if (!newAccessToken) {
                return throwError(() => new Error('No refresh token was available.'));
            }
            this.refreshingAccessToken$.next(newAccessToken);
            return next.handle(this.addAuthorizationHeader(request, newAccessToken));
        }), catchError(() => this.logoutUser()), finalize(() => this.isRefreshing = false));
    }
    logoutUser() {
        // Handle the refresh error.
        this._auth.clearAndRedirect();
        // Return null as data.
        return of(null);
    }
    getAccessToken = (request) => {
        // Get the token based on header.
        if (request.headers.has(this._authHeaderString)) {
            return request.headers.get(this._authHeaderString);
        }
        // Return the default access token.
        return this._auth.session.accessToken;
    };
    addAuthorizationHeader = (request, accessToken = null) => {
        // Remove auth header when we do not have
        // an access token.
        if (!accessToken) {
            return request.clone({
                headers: request.headers.delete(this._authHeaderString)
            });
        }
        // Add the auth header to the request.
        return request.clone({
            setHeaders: { [this._authHeaderString]: `${this._authScheme} ${accessToken}` }
        });
    };
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: AuthInterceptor, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: AuthInterceptor });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: AuthInterceptor, decorators: [{
            type: Injectable
        }] });

function provideAuthConfig(config) {
    return makeEnvironmentProviders([
        { provide: AUTH_CONFIG, useValue: config },
        { provide: HTTP_INTERCEPTORS, useClass: AuthInterceptor, multi: true },
        { useFactory: initializeAuth, provide: APP_INITIALIZER, deps: [Auth], multi: true }
    ]);
}
function initializeAuth(auth) {
    return auth.initialize();
}

class AuthModule {
    static forRoot(config) {
        return {
            ngModule: AuthModule,
            providers: [
                provideAuthConfig(config)
            ]
        };
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: AuthModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
    static ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "18.2.8", ngImport: i0, type: AuthModule, imports: [BbAuthenticated], exports: [BbAuthenticated] });
    static ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: AuthModule });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: AuthModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [BbAuthenticated],
                    exports: [BbAuthenticated]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { AUTH_CONFIG, AUTH_REDIRECT_HANDLER, Auth, AuthEmailProvider, AuthModule, AuthSession, AuthVerifyProvider, BbAuthenticated, JwtHelper, USE_AUTHORIZATION, bbAnonymousGuard, bbAuthenticatedGuard, initializeAuth, provideAuthConfig };
//# sourceMappingURL=bravobit-bb-foundation-auth.mjs.map
