import { isObservable, of, combineLatest, BehaviorSubject, Subscription } from 'rxjs';
import { switchMap, map, delay, shareReplay, distinctUntilChanged, tap, catchError, startWith, debounceTime } from 'rxjs/operators';
import * as i0 from '@angular/core';
import { Directive, TemplateRef, Component, ChangeDetectionStrategy, ViewEncapsulation, ContentChild, Input, numberAttribute, booleanAttribute, Attribute, Optional, HostListener, NgModule } from '@angular/core';
import { AsyncPipe, NgTemplateOutlet } from '@angular/common';
import { BbLocalizeString, BbLocalize, BbLocalizeTemplate } from '@bravobit/bb-foundation/localize';
import { combineLatestMap, getControlValue } from '@bravobit/bb-foundation/rxjs';
import { BbInput, BbIcon, BbFormControl, BbPrefix, BbSpinner, BbAvatar } from '@bravobit/bb-foundation/elements';
import * as i1 from '@angular/forms';
import { ReactiveFormsModule, FormControl } from '@angular/forms';
import { BbTemplate } from '@bravobit/bb-foundation/utils';

class CollectionProvider {
    formatArrayToCollectionData(items) {
        const count = items?.length ?? 1;
        return {
            pageSize: count,
            totalEntries: count,
            pageNumber: 1,
            totalPages: 1,
            data: items
        };
    }
}

class ApiCollectionProvider extends CollectionProvider {
    endpoint;
    httpClient;
    // State.
    extraParams = {};
    extraObjectParams = [];
    constructor(endpoint, httpClient) {
        super();
        this.endpoint = endpoint;
        this.httpClient = httpClient;
    }
    append(name, value) {
        this.extraParams[name] = isObservable(value) ? value : of(value);
        return this;
    }
    appendObject(value) {
        const params = isObservable(value) ? value : of(value);
        this.extraObjectParams.push(params);
    }
    transform(request) {
        const endpoint$ = isObservable(this.endpoint) ? this.endpoint : of(this.endpoint);
        const params = this.getBackendFormat(request);
        const extraParams$ = this.getExtraParams(this.extraParams);
        const extraObjectParams$ = this.getExtraObjectParams(this.extraObjectParams);
        return combineLatest([endpoint$, extraParams$, extraObjectParams$]).pipe(switchMap(([endpoint, extraParams, extraObjectParams]) => {
            return this.httpClient.get(endpoint, {
                params: { ...params, ...extraParams, ...extraObjectParams }
            });
        }), map(response => this.formatToCollectionData(response)));
    }
    getExtraParams(params) {
        const keys = Object.keys(params ?? {});
        if (keys?.length <= 0) {
            return of({});
        }
        const observables$ = keys.map(key => params?.[key]);
        return combineLatest(observables$).pipe(map(data => {
            return data.reduce((previous, current, index) => {
                if (current === null || current === undefined) {
                    return previous;
                }
                const key = keys?.[index];
                return { ...previous, [key]: current };
            }, {});
        }));
    }
    getExtraObjectParams(params$) {
        if (params$?.length <= 0) {
            return of({});
        }
        return combineLatest(params$).pipe(map(data => (data ?? []).reduce((previous, current) => ({ ...previous, ...(current ?? {}) }), {})), map(data => {
            return Object.keys(data ?? {}).reduce((previous, current) => {
                const value = data?.[current] ?? null;
                if (value === null || value === undefined) {
                    return previous;
                }
                return { ...previous, [current]: value };
            }, {});
        }));
    }
    formatToCollectionData(response) {
        if (Array.isArray(response)) {
            return this.formatArrayToCollectionData(response);
        }
        return {
            pageSize: response?.page_size,
            pageNumber: response?.page_number,
            totalPages: response?.total_pages,
            totalEntries: response?.total_entries,
            data: response?.data,
            raw: response
        };
    }
    getBackendFormat(request) {
        const params = {};
        params['page'] = request?.pageNumber ?? 1;
        params['page_size'] = request?.pageSize ?? 25;
        if (request?.sort) {
            const sort = Array.isArray(request?.sort) ? request?.sort : [request?.sort];
            const filteredSort = sort?.filter(item => !!item?.name && !!item?.direction);
            params['sort'] = filteredSort?.map(item => item?.name)?.join(',');
            params['sort_direction'] = filteredSort?.map(item => item?.direction)?.join(',');
        }
        if (request?.query && request?.query?.length > 0 && request?.queryFields && request?.queryFields?.length > 0) {
            params['query'] = request?.query;
            params['query_fields'] = request?.queryFields?.join(',');
        }
        return params;
    }
}

class ArrayCollection {
    value;
    constructor(value) {
        this.value = value;
    }
    search(query, fields) {
        const normalizedQuery = this.normalizeValueAsString(query);
        if (normalizedQuery === null || normalizedQuery?.length <= 0) {
            return this;
        }
        const queryFields = fields ?? [];
        if (queryFields.length <= 0) {
            throw new Error('cannot search without query fields');
        }
        this.value = this.value.filter(item => {
            for (const field of queryFields) {
                const normalizedValue = this.normalizeValueAsString(item?.[field]);
                if (normalizedValue === null) {
                    continue;
                }
                if (normalizedValue.includes(normalizedQuery)) {
                    return true;
                }
            }
            return false;
        });
        return this;
    }
    sortBy(sort, sortables) {
        if (sort === null || sort === undefined) {
            return this;
        }
        const sorts = Array.isArray(sort) ? sort : [sort];
        if (sorts.length <= 0) {
            return this;
        }
        this.value = this.value.sort((a, b) => {
            for (const item of sorts) {
                const type = sortables?.[item?.name] ?? null;
                if (type === null) {
                    throw new Error(`invalid sorting key "${item?.name?.toString()}"`);
                }
                const aValue = a?.[item?.name] ?? null;
                const bValue = b?.[item?.name] ?? null;
                const direction = item?.direction ?? null;
                if (type === 'string') {
                    const comparison = String(aValue).localeCompare(String(bValue));
                    if (comparison !== 0) {
                        return direction === 'asc' ? comparison : -comparison;
                    }
                }
                else if (type === 'number') {
                    if (aValue < bValue)
                        return direction === 'asc' ? -1 : 1;
                    if (aValue > bValue)
                        return direction === 'asc' ? 1 : -1;
                }
            }
            return 0;
        });
        return this;
    }
    toPage(pageNumber, pageSize) {
        const totalEntries = this.value?.length ?? 0;
        const totalPages = Math.max(1, Math.ceil(totalEntries / pageSize));
        const startIndex = (pageNumber - 1) * pageSize;
        const endIndex = startIndex + pageSize;
        const data = this.value?.slice(startIndex, endIndex) ?? [];
        return { totalPages, totalEntries, pageSize, pageNumber, data };
    }
    toArray() {
        return this.value;
    }
    normalizeValueAsString(value) {
        if (value === null || value === undefined) {
            return null;
        }
        return value
            .toString()
            .toLowerCase()
            .normalize('NFD')
            .replace(/[\u0300-\u036f]/g, '');
    }
}

class LocalCollectionProvider extends CollectionProvider {
    _options;
    _items$ = new BehaviorSubject([]);
    constructor(items, _options) {
        super();
        this._options = _options;
        this.update(items);
    }
    update(value) {
        this._items$.next(value);
    }
    transform(request) {
        return this._items$.pipe(map(items => {
            return new ArrayCollection(items)
                .search(request?.query, request.queryFields)
                .sortBy(request?.sort, this._options?.sortables ?? {})
                .toPage(request?.pageNumber, request?.pageSize);
        }), delay(1));
    }
}

class BbCollectionsDataContext {
    items;
    loading;
    constructor(items, loading) {
        this.items = items;
        this.loading = loading;
    }
}
class BbCollectionsData {
    template;
    constructor(template) {
        this.template = template;
    }
    static ngTemplateContextGuard(dir, ctx) {
        return true;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsData, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.8", type: BbCollectionsData, isStandalone: true, selector: "[bbCollectionsData]", ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsData, decorators: [{
            type: Directive,
            args: [{
                    selector: '[bbCollectionsData]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: i0.TemplateRef }] });

class BbCollectionsDataRowContext {
    item;
    $implicit;
    constructor(item) {
        this.item = item;
        this.$implicit = item;
    }
}
class BbCollectionsDataRow {
    template;
    constructor(template) {
        this.template = template;
    }
    static ngTemplateContextGuard(dir, ctx) {
        return true;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsDataRow, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.8", type: BbCollectionsDataRow, isStandalone: true, selector: "[bbCollectionsDataRow]", ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsDataRow, decorators: [{
            type: Directive,
            args: [{
                    selector: '[bbCollectionsDataRow]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: i0.TemplateRef }] });

class BbCollectionsEmptyContext {
    query;
    loading;
    constructor(query, loading) {
        this.query = query;
        this.loading = loading;
    }
}
class BbCollectionsEmpty {
    template;
    constructor(template) {
        this.template = template;
    }
    static ngTemplateContextGuard(dir, ctx) {
        return true;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsEmpty, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.8", type: BbCollectionsEmpty, isStandalone: true, selector: "[bbCollectionsEmpty]", ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsEmpty, decorators: [{
            type: Directive,
            args: [{
                    selector: '[bbCollectionsEmpty]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: i0.TemplateRef }] });

class BbCollectionsErrorContext {
    error;
    loading;
    constructor(error, loading) {
        this.error = error;
        this.loading = loading;
    }
}
class BbCollectionsError {
    template;
    constructor(template) {
        this.template = template;
    }
    static ngTemplateContextGuard(dir, ctx) {
        return true;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsError, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.8", type: BbCollectionsError, isStandalone: true, selector: "[bbCollectionsError]", ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsError, decorators: [{
            type: Directive,
            args: [{
                    selector: '[bbCollectionsError]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: i0.TemplateRef }] });

class BbCollectionsInitialContext {
}
class BbCollectionsInitial {
    template;
    constructor(template) {
        this.template = template;
    }
    static ngTemplateContextGuard(dir, ctx) {
        return true;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsInitial, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.8", type: BbCollectionsInitial, isStandalone: true, selector: "[bbCollectionsInitial]", ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsInitial, decorators: [{
            type: Directive,
            args: [{
                    selector: '[bbCollectionsInitial]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: i0.TemplateRef }] });

class BbCollectionsItemContext {
    item;
    $implicit;
    constructor(item) {
        this.item = item;
        this.$implicit = item;
    }
}
class BbCollectionsItem {
    template;
    constructor(template) {
        this.template = template;
    }
    static ngTemplateContextGuard(dir, ctx) {
        return true;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsItem, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.8", type: BbCollectionsItem, isStandalone: true, selector: "[bbCollectionsItem]", ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsItem, decorators: [{
            type: Directive,
            args: [{
                    selector: '[bbCollectionsItem]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: i0.TemplateRef }] });

class BbCollectionsViewer {
    // Content.
    dataTemplate;
    emptyTemplate;
    errorTemplate;
    initialTemplate;
    // Inputs.
    collection = null;
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsViewer, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "18.2.8", type: BbCollectionsViewer, isStandalone: true, selector: "[bb-collections-viewer]", inputs: { collection: "collection" }, host: { classAttribute: "bb-collections-viewer" }, queries: [{ propertyName: "dataTemplate", first: true, predicate: BbCollectionsData, descendants: true, read: TemplateRef }, { propertyName: "emptyTemplate", first: true, predicate: BbCollectionsEmpty, descendants: true, read: TemplateRef }, { propertyName: "errorTemplate", first: true, predicate: BbCollectionsError, descendants: true, read: TemplateRef }, { propertyName: "initialTemplate", first: true, predicate: BbCollectionsInitial, descendants: true, read: TemplateRef }], ngImport: i0, template: "@if (collection?.data$ | async; as data) {\n    @switch (data?.state) {\n        @case ('initial') {\n            <ng-container *ngTemplateOutlet=\"initialTemplate\"></ng-container>\n        }\n        @case ('empty') {\n            <ng-container\n                *ngTemplateOutlet=\"emptyTemplate; context: {query: data?.request?.query, loading: data?.loading}\"></ng-container>\n        }\n        @case ('error') {\n            <ng-container\n                *ngTemplateOutlet=\"errorTemplate; context: {error: data?.error, loading: data?.loading}\"></ng-container>\n        }\n        @default {\n            <ng-container\n                *ngTemplateOutlet=\"dataTemplate; context: {items: data?.response?.data, loading: data?.loading}\"></ng-container>\n        }\n    }\n}\n", styles: [".bb-collections-viewer{display:block}\n"], dependencies: [{ kind: "pipe", type: AsyncPipe, name: "async" }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsViewer, decorators: [{
            type: Component,
            args: [{ selector: '[bb-collections-viewer]', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, host: { 'class': 'bb-collections-viewer' }, preserveWhitespaces: false, standalone: true, imports: [AsyncPipe, NgTemplateOutlet], template: "@if (collection?.data$ | async; as data) {\n    @switch (data?.state) {\n        @case ('initial') {\n            <ng-container *ngTemplateOutlet=\"initialTemplate\"></ng-container>\n        }\n        @case ('empty') {\n            <ng-container\n                *ngTemplateOutlet=\"emptyTemplate; context: {query: data?.request?.query, loading: data?.loading}\"></ng-container>\n        }\n        @case ('error') {\n            <ng-container\n                *ngTemplateOutlet=\"errorTemplate; context: {error: data?.error, loading: data?.loading}\"></ng-container>\n        }\n        @default {\n            <ng-container\n                *ngTemplateOutlet=\"dataTemplate; context: {items: data?.response?.data, loading: data?.loading}\"></ng-container>\n        }\n    }\n}\n", styles: [".bb-collections-viewer{display:block}\n"] }]
        }], propDecorators: { dataTemplate: [{
                type: ContentChild,
                args: [BbCollectionsData, { read: TemplateRef, static: false }]
            }], emptyTemplate: [{
                type: ContentChild,
                args: [BbCollectionsEmpty, { read: TemplateRef, static: false }]
            }], errorTemplate: [{
                type: ContentChild,
                args: [BbCollectionsError, { read: TemplateRef, static: false }]
            }], initialTemplate: [{
                type: ContentChild,
                args: [BbCollectionsInitial, { read: TemplateRef, static: false }]
            }], collection: [{
                type: Input
            }] } });

class BbCollectionsPager {
    set maxAmountOfButtons(value) {
        this._maxAmountOfButtons$.next(Math.max(1, value));
    }
    get maxAmountOfButtons() {
        return this._maxAmountOfButtons$.getValue();
    }
    set collection(collection) {
        this._collection = collection;
        this.observeCollectionChanges();
    }
    get collection() {
        return this._collection;
    }
    // Data.
    data$;
    // State and subscriptions.
    _collection = null;
    _subscription = null;
    _cache$ = new BehaviorSubject({});
    _maxAmountOfButtons$ = new BehaviorSubject(3);
    ngOnInit() {
        this.setData();
    }
    ngOnDestroy() {
        this._subscription?.unsubscribe();
    }
    setPage(pageNumber, totalPages) {
        // Calculate the new value.
        const newValue = Math.min(totalPages, Math.max(1, pageNumber));
        // Validate the value is actually different.
        const currentValue = this._collection?.pageNumberControl?.value;
        if (currentValue === newValue) {
            return;
        }
        this._collection?.pageNumberControl?.setValue(pageNumber);
    }
    setData() {
        this.data$ = combineLatestMap({
            cache: this._cache$,
            maxAmountOfButtons: this._maxAmountOfButtons$
        }).pipe(map(({ cache, maxAmountOfButtons }) => {
            const pageSize = cache?.pageSize ?? 1;
            const pageNumber = cache?.pageNumber ?? 1;
            const totalEntries = Math.max(0, cache?.totalEntries ?? 0);
            const start = Math.max(0, Math.min((pageSize * (pageNumber - 1)) + 1, totalEntries));
            const end = Math.max(0, Math.min((start - 1) + pageSize, totalEntries));
            const totalPages = Math.max(0, cache?.totalPages ?? 1);
            const pages = this.filterActivePages(pageNumber, totalPages, maxAmountOfButtons);
            return { start, end, totalEntries, totalPages, pages, pageNumber };
        }), shareReplay({ refCount: true, bufferSize: 1 }));
    }
    observeCollectionChanges() {
        if (this._subscription) {
            this._subscription?.unsubscribe();
        }
        if (!this._collection) {
            return;
        }
        this._subscription = this._collection.raw$.subscribe(data => {
            if (data) {
                this._cache$.next(data);
            }
        });
    }
    calculatePages(pageNumber, totalPages, maxAmountOfButtons) {
        const min = 1;
        const max = totalPages;
        const amount = Math.max(1, maxAmountOfButtons);
        const start = Math.max(0, pageNumber - Math.floor(amount / 2));
        const data = Array(amount)
            .fill(0)
            .map((_, index) => start + index);
        // Validate if the minimum value less than
        // the first value.
        if (data[0] < min) {
            return data.map(item => item + 1);
        }
        // Calculate the difference with the max
        // if this value is higher than 0 we should minus it.
        const lastValueInArray = data[data.length - 1];
        const differenceWithMax = lastValueInArray - max;
        if (differenceWithMax > 0) {
            return data.map(item => item - differenceWithMax);
        }
        // Just return the default data.
        return data;
    }
    filterActivePages(pageNumber, totalPages, maxAmountOfButtons) {
        return this.calculatePages(pageNumber, totalPages, maxAmountOfButtons)
            .filter(page => page > 0 && page <= totalPages)
            .map(page => ({ page, active: pageNumber === page }));
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsPager, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "18.2.8", type: BbCollectionsPager, isStandalone: true, selector: "bb-collections-pager", inputs: { maxAmountOfButtons: ["maxAmountOfButtons", "maxAmountOfButtons", numberAttribute], collection: "collection" }, host: { classAttribute: "bb-collections-pager" }, ngImport: i0, template: "@if (data$ | async; as data) {\n    <div [bb-localize-string]=\"'collection.showing_label' | bbLocalize\"\n         class=\"bb-collections-pager-text\">\n        <strong *bbLocalizeTemplate=\"'start'\">{{ data?.start }}</strong>\n        <strong *bbLocalizeTemplate=\"'end'\">{{ data?.end }}</strong>\n        <strong *bbLocalizeTemplate=\"'total'\">{{ data?.totalEntries }}</strong>\n    </div>\n\n    <div class=\"bb-collections-pager-container\">\n        <ul class=\"bb-collections-pager-list\">\n            <li class=\"bb-collections-pager-item\"\n                title=\"Previous page\">\n                <button [disabled]=\"data?.pageNumber <= 1\"\n                        (click)=\"setPage(data?.pageNumber - 1, data?.totalPages)\"\n                        class=\"bb-collections-pager-button arrow first\"\n                        type=\"button\">\n                    &#8592;\n                </button>\n            </li>\n            @for (item of data?.pages; track item?.page) {\n                <li [title]=\"'Page ' + item?.page\"\n                    class=\"bb-collections-pager-item\">\n                    <button [class.active]=\"item?.active\"\n                            (click)=\"setPage(item?.page, data?.totalPages)\"\n                            class=\"bb-collections-pager-button\"\n                            type=\"button\">\n                        {{ item?.page }}\n                    </button>\n                </li>\n            }\n            <li class=\"bb-collections-pager-item\"\n                title=\"Next page\">\n                <button [disabled]=\"data?.pageNumber >= data?.totalPages\"\n                        (click)=\"setPage(data?.pageNumber + 1, data?.totalPages)\"\n                        class=\"bb-collections-pager-button arrow last\"\n                        type=\"button\">\n                    &#8594;\n                </button>\n            </li>\n        </ul>\n    </div>\n}\n", styles: [".bb-collections-pager{display:flex;align-items:center;font-size:.875rem;flex-direction:column-reverse}.bb-collections-pager-text{display:block;margin-top:.75rem}.bb-collections-pager-text>strong{font-weight:500}.bb-collections-pager-container{width:100%;display:flex;overflow-x:auto;align-items:center;justify-content:center}.bb-collections-pager-list{display:flex;align-items:center;border-radius:.5rem;background-color:#fff;border:1px solid hsl(213,8%,74%);box-shadow:0 .375rem .375rem -.375rem #0000001a}.bb-collections-pager-item:not(:last-child){border-right:1px solid hsl(213,8%,74%)}.bb-collections-pager-button{height:2rem;border:none;display:flex;appearance:none;min-height:2rem;min-width:2.5rem;-webkit-user-select:none;user-select:none;padding:0 .25rem;align-items:center;justify-content:center;background-color:transparent}.bb-collections-pager-button:hover,.bb-collections-pager-button:focus,.bb-collections-pager-button:active,.bb-collections-pager-button.active{font-weight:400;color:var(--bb-collections-pager-color)}.bb-collections-pager-button:hover,.bb-collections-pager-button:focus{background-color:var(--bb-collections-pager-hover-background-color)}.bb-collections-pager-button:active,.bb-collections-pager-button.active{background-color:var(--bb-collections-pager-active-background-color)}.bb-collections-pager-button:not(.arrow):focus,.bb-collections-pager-button:not(.arrow):active{box-shadow:var(--bb-collections-pager-focus-box-shadow)}.bb-collections-pager-button:not(.arrow):focus,.bb-collections-pager-button:not(.arrow):active,.bb-collections-pager-button:not(.arrow).active{outline:1px solid var(--bb-collections-pager-outline-color)}.bb-collections-pager-button.arrow{font-weight:500}.bb-collections-pager-button.arrow.first{border-top-left-radius:calc(.5rem - 1px);border-bottom-left-radius:calc(.5rem - 1px)}.bb-collections-pager-button.arrow.last{border-top-right-radius:calc(.5rem - 1px);border-bottom-right-radius:calc(.5rem - 1px)}.bb-collections-pager-button:disabled{color:#111;outline:none;opacity:.25;cursor:default;pointer-events:none;background-color:transparent}@media only screen and (min-width: 768px){.bb-collections-pager{flex-wrap:wrap;align-items:center;flex-direction:row;justify-content:space-between}.bb-collections-pager-text{margin-top:0}.bb-collections-pager-container{width:auto;overflow-x:unset}}\n"], dependencies: [{ kind: "pipe", type: AsyncPipe, name: "async" }, { kind: "component", type: BbLocalizeString, selector: "[bb-localize-string]", inputs: ["substitutions", "bb-localize-string"] }, { kind: "pipe", type: BbLocalize, name: "bbLocalize" }, { kind: "directive", type: BbLocalizeTemplate, selector: "[bbLocalizeTemplate]", inputs: ["bbLocalizeTemplate"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsPager, decorators: [{
            type: Component,
            args: [{ selector: 'bb-collections-pager', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, host: { 'class': 'bb-collections-pager' }, preserveWhitespaces: false, standalone: true, imports: [AsyncPipe, BbLocalizeString, BbLocalize, BbLocalizeTemplate], template: "@if (data$ | async; as data) {\n    <div [bb-localize-string]=\"'collection.showing_label' | bbLocalize\"\n         class=\"bb-collections-pager-text\">\n        <strong *bbLocalizeTemplate=\"'start'\">{{ data?.start }}</strong>\n        <strong *bbLocalizeTemplate=\"'end'\">{{ data?.end }}</strong>\n        <strong *bbLocalizeTemplate=\"'total'\">{{ data?.totalEntries }}</strong>\n    </div>\n\n    <div class=\"bb-collections-pager-container\">\n        <ul class=\"bb-collections-pager-list\">\n            <li class=\"bb-collections-pager-item\"\n                title=\"Previous page\">\n                <button [disabled]=\"data?.pageNumber <= 1\"\n                        (click)=\"setPage(data?.pageNumber - 1, data?.totalPages)\"\n                        class=\"bb-collections-pager-button arrow first\"\n                        type=\"button\">\n                    &#8592;\n                </button>\n            </li>\n            @for (item of data?.pages; track item?.page) {\n                <li [title]=\"'Page ' + item?.page\"\n                    class=\"bb-collections-pager-item\">\n                    <button [class.active]=\"item?.active\"\n                            (click)=\"setPage(item?.page, data?.totalPages)\"\n                            class=\"bb-collections-pager-button\"\n                            type=\"button\">\n                        {{ item?.page }}\n                    </button>\n                </li>\n            }\n            <li class=\"bb-collections-pager-item\"\n                title=\"Next page\">\n                <button [disabled]=\"data?.pageNumber >= data?.totalPages\"\n                        (click)=\"setPage(data?.pageNumber + 1, data?.totalPages)\"\n                        class=\"bb-collections-pager-button arrow last\"\n                        type=\"button\">\n                    &#8594;\n                </button>\n            </li>\n        </ul>\n    </div>\n}\n", styles: [".bb-collections-pager{display:flex;align-items:center;font-size:.875rem;flex-direction:column-reverse}.bb-collections-pager-text{display:block;margin-top:.75rem}.bb-collections-pager-text>strong{font-weight:500}.bb-collections-pager-container{width:100%;display:flex;overflow-x:auto;align-items:center;justify-content:center}.bb-collections-pager-list{display:flex;align-items:center;border-radius:.5rem;background-color:#fff;border:1px solid hsl(213,8%,74%);box-shadow:0 .375rem .375rem -.375rem #0000001a}.bb-collections-pager-item:not(:last-child){border-right:1px solid hsl(213,8%,74%)}.bb-collections-pager-button{height:2rem;border:none;display:flex;appearance:none;min-height:2rem;min-width:2.5rem;-webkit-user-select:none;user-select:none;padding:0 .25rem;align-items:center;justify-content:center;background-color:transparent}.bb-collections-pager-button:hover,.bb-collections-pager-button:focus,.bb-collections-pager-button:active,.bb-collections-pager-button.active{font-weight:400;color:var(--bb-collections-pager-color)}.bb-collections-pager-button:hover,.bb-collections-pager-button:focus{background-color:var(--bb-collections-pager-hover-background-color)}.bb-collections-pager-button:active,.bb-collections-pager-button.active{background-color:var(--bb-collections-pager-active-background-color)}.bb-collections-pager-button:not(.arrow):focus,.bb-collections-pager-button:not(.arrow):active{box-shadow:var(--bb-collections-pager-focus-box-shadow)}.bb-collections-pager-button:not(.arrow):focus,.bb-collections-pager-button:not(.arrow):active,.bb-collections-pager-button:not(.arrow).active{outline:1px solid var(--bb-collections-pager-outline-color)}.bb-collections-pager-button.arrow{font-weight:500}.bb-collections-pager-button.arrow.first{border-top-left-radius:calc(.5rem - 1px);border-bottom-left-radius:calc(.5rem - 1px)}.bb-collections-pager-button.arrow.last{border-top-right-radius:calc(.5rem - 1px);border-bottom-right-radius:calc(.5rem - 1px)}.bb-collections-pager-button:disabled{color:#111;outline:none;opacity:.25;cursor:default;pointer-events:none;background-color:transparent}@media only screen and (min-width: 768px){.bb-collections-pager{flex-wrap:wrap;align-items:center;flex-direction:row;justify-content:space-between}.bb-collections-pager-text{margin-top:0}.bb-collections-pager-container{width:auto;overflow-x:unset}}\n"] }]
        }], propDecorators: { maxAmountOfButtons: [{
                type: Input,
                args: [{ transform: numberAttribute }]
            }], collection: [{
                type: Input,
                args: [{ required: true }]
            }] } });

class BbCollectionsActions {
    // Inputs.
    collection = null;
    queryPlaceholder = null;
    standalone = false;
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsActions, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "16.1.0", version: "18.2.8", type: BbCollectionsActions, isStandalone: true, selector: "bb-collections-actions", inputs: { collection: "collection", queryPlaceholder: "queryPlaceholder", standalone: ["standalone", "standalone", booleanAttribute] }, host: { properties: { "class.standalone": "standalone" }, classAttribute: "bb-collections-actions" }, ngImport: i0, template: "<div class=\"bb-collections-actions-row\">\n    <bb-form-control class=\"bb-collections-actions-search\">\n        <bb-icon bbPrefix\n                 src=\"material:search\"></bb-icon>\n        <input [formControl]=\"collection?.queryControl\"\n               [placeholder]=\"queryPlaceholder ?? ('collection.search_label' | bbLocalize)\"\n               bbInput\n               autocomplete=\"off\"\n               type=\"search\">\n    </bb-form-control>\n    <ng-content select=\"[data-collections-action]\"></ng-content>\n</div>\n<ng-content></ng-content>\n", styles: [".bb-collections-actions{gap:.75rem;display:flex;color:#111;font-size:.875rem;padding:.75rem 1rem;flex-direction:column;border-bottom:1px solid hsl(213,8%,90%)}.bb-collections-actions.standalone{border-bottom:none;border-radius:.5rem;margin-bottom:1.5rem;background-color:#fff;box-shadow:0 .375rem .375rem -.375rem #0000001a}.bb-collections-actions-row,[data-collections-row]{gap:.75rem;display:flex;flex-direction:column}.bb-collections-actions-search{flex:1}@media only screen and (min-width: 768px){.bb-collections-actions-row,[data-collections-row]{flex-direction:row}.bb-collections-actions-row.end,[data-collections-row].end{justify-content:flex-end}}\n"], dependencies: [{ kind: "ngmodule", type: ReactiveFormsModule }, { kind: "directive", type: i1.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { kind: "directive", type: i1.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i1.FormControlDirective, selector: "[formControl]", inputs: ["formControl", "disabled", "ngModel"], outputs: ["ngModelChange"], exportAs: ["ngForm"] }, { kind: "directive", type: BbInput, selector: "input[bbInput],textarea[bbInput],select[bbInput]", inputs: ["required"] }, { kind: "pipe", type: BbLocalize, name: "bbLocalize" }, { kind: "component", type: BbIcon, selector: "bb-icon", inputs: ["alt", "size", "ariaHidden", "unit", "src"] }, { kind: "component", type: BbFormControl, selector: "bb-form-control", inputs: ["label", "hint", "grouped", "hideErrors"] }, { kind: "directive", type: BbPrefix, selector: "[bbPrefix]" }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsActions, decorators: [{
            type: Component,
            args: [{ selector: 'bb-collections-actions', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, host: {
                        'class': 'bb-collections-actions',
                        '[class.standalone]': 'standalone'
                    }, preserveWhitespaces: false, standalone: true, imports: [
                        ReactiveFormsModule,
                        BbInput,
                        BbLocalize,
                        BbIcon,
                        BbFormControl,
                        BbPrefix
                    ], template: "<div class=\"bb-collections-actions-row\">\n    <bb-form-control class=\"bb-collections-actions-search\">\n        <bb-icon bbPrefix\n                 src=\"material:search\"></bb-icon>\n        <input [formControl]=\"collection?.queryControl\"\n               [placeholder]=\"queryPlaceholder ?? ('collection.search_label' | bbLocalize)\"\n               bbInput\n               autocomplete=\"off\"\n               type=\"search\">\n    </bb-form-control>\n    <ng-content select=\"[data-collections-action]\"></ng-content>\n</div>\n<ng-content></ng-content>\n", styles: [".bb-collections-actions{gap:.75rem;display:flex;color:#111;font-size:.875rem;padding:.75rem 1rem;flex-direction:column;border-bottom:1px solid hsl(213,8%,90%)}.bb-collections-actions.standalone{border-bottom:none;border-radius:.5rem;margin-bottom:1.5rem;background-color:#fff;box-shadow:0 .375rem .375rem -.375rem #0000001a}.bb-collections-actions-row,[data-collections-row]{gap:.75rem;display:flex;flex-direction:column}.bb-collections-actions-search{flex:1}@media only screen and (min-width: 768px){.bb-collections-actions-row,[data-collections-row]{flex-direction:row}.bb-collections-actions-row.end,[data-collections-row].end{justify-content:flex-end}}\n"] }]
        }], propDecorators: { collection: [{
                type: Input,
                args: [{ required: true }]
            }], queryPlaceholder: [{
                type: Input
            }], standalone: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }] } });

class BbCollectionsTable {
    // Readonly data.
    collection$ = new BehaviorSubject(null);
    // Views.
    dataRowTemplate;
    // Inputs.
    queryPlaceholder = null;
    emptyContent = null;
    noResultsContent = null;
    errorContent = null;
    trackByProperty = 'id';
    includeVerticalLines = false;
    hideActions = false;
    hidePager = false;
    set collection(collection) {
        this.collection$.next(collection);
    }
    get collection() {
        return this.collection$.getValue();
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsTable, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "18.2.8", type: BbCollectionsTable, isStandalone: true, selector: "bb-collections-table", inputs: { queryPlaceholder: "queryPlaceholder", emptyContent: "emptyContent", noResultsContent: "noResultsContent", errorContent: "errorContent", trackByProperty: "trackByProperty", includeVerticalLines: ["includeVerticalLines", "includeVerticalLines", booleanAttribute], hideActions: ["hideActions", "hideActions", booleanAttribute], hidePager: ["hidePager", "hidePager", booleanAttribute], collection: "collection" }, host: { properties: { "class.include-vertical-lines": "includeVerticalLines" }, classAttribute: "bb-collections-table" }, queries: [{ propertyName: "dataRowTemplate", first: true, predicate: BbCollectionsDataRow, descendants: true, read: TemplateRef }], ngImport: i0, template: "@if (!hideActions) {\n    <bb-collections-actions [collection]=\"collection\"\n                            [queryPlaceholder]=\"queryPlaceholder\">\n        <ng-content ngProjectAs=\"[data-collections-action]\"\n                    select=\"[data-collections-action]\"></ng-content>\n        <ng-content ngProjectAs=\"[data-collections-row]\"\n                    select=\"[data-collections-row]\"></ng-content>\n    </bb-collections-actions>\n}\n\n<div [class.loading]=\"collection?.loading$ | async\"\n     class=\"bb-collections-table-wrapper\">\n    <div [collection]=\"collection\"\n         class=\"bb-collections-table-container\"\n         bb-collections-viewer>\n        @if (collection?.loading$ | async) {\n            <div class=\"bb-collections-table-loading\"></div>\n        }\n        <table *bbCollectionsData=\"let items = items; let loading = loading\">\n            <ng-content select=\"thead\"></ng-content>\n            <tbody>\n                @for (item of items; track (trackByProperty === null ? $index : item[trackByProperty])) {\n                    <ng-container\n                        *ngTemplateOutlet=\"dataRowTemplate; context: {$implicit: item, item: item}\"></ng-container>\n                }\n            </tbody>\n            <ng-content select=\"tfoot\"></ng-content>\n        </table>\n        <div *bbCollectionsEmpty=\"let query = query\"\n             class=\"bb-collections-table-info\">\n            <div class=\"bb-collections-table-image empty\"></div>\n            @if (query?.length > 0) {\n                <ng-template [bbTemplate]=\"noResultsContent\">\n                    <p [bb-localize-string]=\"$any(noResultsContent) ?? ('collection.no_results_label' | bbLocalize)\"\n                       class=\"bb-collections-table-info-text\">\n                        <strong *bbLocalizeTemplate=\"'query'\">{{ query ?? '...' }}</strong>\n                    </p>\n                </ng-template>\n            } @else {\n                <ng-template [bbTemplate]=\"emptyContent\">\n                    <p class=\"bb-collections-table-info-text\">\n                        {{ $any(emptyContent) ?? ('collection.empty_label' | bbLocalize) }}\n                    </p>\n                </ng-template>\n            }\n        </div>\n        <div *bbCollectionsError=\"let error = error\"\n             class=\"bb-collections-table-info\">\n            <div class=\"bb-collections-table-image error\"></div>\n            <ng-template [bbTemplate]=\"errorContent\">\n                <p class=\"bb-collections-table-info-text\">\n                    {{ $any(errorContent) ?? ('collection.error_label' | bbLocalize) }}\n                </p>\n            </ng-template>\n        </div>\n        <div *bbCollectionsInitial\n             class=\"bb-collections-table-info\">\n            <bb-spinner class=\"block\"></bb-spinner>\n        </div>\n    </div>\n</div>\n\n@if (!hidePager) {\n    <footer class=\"bb-collections-table-footer\">\n        <bb-collections-pager [collection]=\"collection\"></bb-collections-pager>\n    </footer>\n}\n", styles: ["@charset \"UTF-8\";.bb-collections-table{color:#111;display:block;position:relative;font-size:.875rem;border-radius:.5rem;background-color:#fff;box-shadow:0 .375rem .375rem -.375rem #0000001a}.bb-collections-table>*:first-child{border-top-left-radius:.5rem;border-top-right-radius:.5rem}.bb-collections-table>*:last-child{border-bottom-left-radius:.5rem;border-bottom-right-radius:.5rem}.bb-collections-table.include-vertical-lines td:not(:last-child),.bb-collections-table.include-vertical-lines th:not(:last-child){border-right:1px solid hsl(213,8%,90%)}.bb-collections-table-footer{padding:.75rem 1rem;background-color:#fbf9f9;border-top:1px solid hsl(213,8%,90%)}.bb-collections-table-wrapper{overflow:hidden;position:relative}.bb-collections-table-wrapper.loading:after{top:0;left:0;right:0;z-index:1;opacity:0;content:\"\";display:block;height:.25rem;position:absolute;will-change:left,right;background-color:#1565c1;animation:collectionsFadeIn .12s cubic-bezier(.65,.815,.735,.395) .4s forwards,collectionsIndeterminate 2.1s cubic-bezier(.65,.815,.735,.395) .4s infinite}.bb-collections-table-container{overflow-x:auto;position:relative}.bb-collections-table-container.loading>table>tbody,.bb-collections-table-container.loading>.bb-collections-table-info{animation:collectionsBlink 1s linear infinite}.bb-collections-table-container>table{width:100%}.bb-collections-table-container>table th,.bb-collections-table-container>table td{padding:1rem;text-align:left;font-weight:400;font-size:.875rem}.bb-collections-table-container>table th.fit,.bb-collections-table-container>table td.fit{width:1%;white-space:nowrap}.bb-collections-table-container>table th.left,.bb-collections-table-container>table td.left{text-align:left}.bb-collections-table-container>table th.right,.bb-collections-table-container>table td.right{text-align:right}.bb-collections-table-container>table>thead{background-color:#fbf9f9;border-bottom:1px solid hsl(213,8%,90%)}.bb-collections-table-container>table>thead>tr>th{padding:1rem;color:#525252}.bb-collections-table-container>table>tbody>tr>td{color:#111}.bb-collections-table-container>table>tbody>tr:not(:last-child){border-bottom:1px solid hsl(213,8%,90%)}.bb-collections-table-info{padding:3rem 1.5rem}.bb-collections-table-info-text{color:#525252;margin:0 auto;display:block;line-height:2;max-width:30rem;text-align:center;font-size:.875rem}.bb-collections-table-info-text>strong{font-weight:500}.bb-collections-table-info-text>strong:before{content:\"\\201c\"}.bb-collections-table-info-text>strong:after{content:\"\\201d\"}.bb-collections-table-image{width:100%;height:auto;display:flex;max-width:7.5rem;margin:0 auto 1.5rem;background-size:contain;background-repeat:no-repeat;background-position:center center}.bb-collections-table-image.empty{aspect-ratio:154/132;mask-image:linear-gradient(to bottom,black 25%,transparent 100%);background-image:url('data:image/svg+xml,%3Csvg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 154 132\"%3E%3Cg fill=\"none\" fill-rule=\"evenodd\"%3E%3Crect width=\"154\" height=\"40\" fill=\"%23FFF\" fill-rule=\"nonzero\" rx=\"8\"/%3E%3Crect width=\"24\" height=\"24\" x=\"8\" y=\"8\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"4\"/%3E%3Crect width=\"60\" height=\"6\" x=\"40\" y=\"11\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"3\"/%3E%3Crect width=\"106\" height=\"6\" x=\"40\" y=\"23\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"3\"/%3E%3Crect width=\"153\" height=\"39\" x=\".5\" y=\".5\" stroke=\"%23E5E7EB\" rx=\"7.5\"/%3E%3Cg transform=\"translate(0 46)\"%3E%3Crect width=\"154\" height=\"40\" fill=\"%23FFF\" fill-rule=\"nonzero\" rx=\"8\"/%3E%3Crect width=\"24\" height=\"24\" x=\"8\" y=\"8\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"4\"/%3E%3Crect width=\"60\" height=\"6\" x=\"40\" y=\"11\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"3\"/%3E%3Crect width=\"106\" height=\"6\" x=\"40\" y=\"23\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"3\"/%3E%3Crect width=\"153\" height=\"39\" x=\".5\" y=\".5\" stroke=\"%23E5E7EB\" rx=\"7.5\"/%3E%3C/g%3E%3Cg transform=\"translate(0 92)\"%3E%3Crect width=\"154\" height=\"40\" fill=\"%23FFF\" fill-rule=\"nonzero\" rx=\"8\"/%3E%3Crect width=\"24\" height=\"24\" x=\"8\" y=\"8\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"4\"/%3E%3Crect width=\"60\" height=\"6\" x=\"40\" y=\"11\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"3\"/%3E%3Crect width=\"106\" height=\"6\" x=\"40\" y=\"23\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"3\"/%3E%3Crect width=\"153\" height=\"39\" x=\".5\" y=\".5\" stroke=\"%23E5E7EB\" rx=\"7.5\"/%3E%3C/g%3E%3C/g%3E%3C/svg%3E')}.bb-collections-table-image.error{max-width:4rem;aspect-ratio:1;background-image:url('data:image/svg+xml,%3Csvg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 132 132\"%3E%3Cpath fill=\"%23C23934\" d=\"M65.958253 98.6024096c.8892771 0 1.6486747-.3008433 2.2781928-.9025301.6295181-.6016867.9442771-1.3471687.9442771-2.2364458 0-.8892771-.3008434-1.6486747-.9025301-2.2781927-.6016868-.6295181-1.3471687-.9442771-2.2364458-.9442771-.8892771 0-1.6486747.3008433-2.2781928.9025301-.6295181.6016867-.9442771 1.3471687-.9442771 2.2364458 0 .8892771.3008434 1.6486747.9025301 2.2781927.6016868.6295181 1.3471687.9442771 2.2364458.9442771Zm-1.9462048-21.4698795h3.9759036V29.4216867h-3.9759036v47.7108434ZM66.0894578 132c-9.0716867 0-17.6205421-1.715602-25.6465662-5.146807-8.0260241-3.431205-15.0547591-8.159217-21.0862049-14.184036-6.032771-6.02482-10.76674694-13.0416269-14.20192766-21.0504221C1.71825301 83.6086145 0 75.0504819 0 65.9443373 0 56.8368675 1.71560241 48.303253 5.14680723 40.343494c3.43120482-7.9597591 8.15921687-14.9553615 14.18403617-20.9868073 6.0248193-6.032771 13.0416265-10.76674694 21.0504217-14.20192766C48.3913855 1.71825301 56.9495181 0 66.0556627 0 75.1631325 0 83.696747 1.71560241 91.656506 5.14680723c7.9597591 3.43120482 14.955361 8.15921687 20.986807 14.18403617 6.032771 6.0248193 10.766747 13.0263855 14.201928 21.0046988C130.281747 48.3138554 132 56.8388554 132 65.9105422c0 9.0716867-1.715602 17.6205421-5.146807 25.6465662-3.431205 8.0260241-8.159217 15.0547586-14.184036 21.0862046-6.02482 6.032771-13.0263859 10.766747-21.0046992 14.201928C83.6861446 130.281747 75.1611446 132 66.0894578 132ZM66 128.024096c17.2289157 0 31.873494-6.03012 43.933735-18.090361C121.993976 97.873494 128.024096 83.2289157 128.024096 66c0-17.2289157-6.03012-31.873494-18.090361-43.9337349C97.873494 10.0060241 83.2289157 3.97590361 66 3.97590361c-17.2289157 0-31.873494 6.03012049-43.9337349 18.09036149C10.0060241 34.126506 3.97590361 48.7710843 3.97590361 66c0 17.2289157 6.03012049 31.873494 18.09036149 43.933735C34.126506 121.993976 48.7710843 128.024096 66 128.024096Z\"/%3E%3C/svg%3E')}@keyframes collectionsFadeIn{0%{opacity:0}to{opacity:1}}@keyframes collectionsIndeterminate{0%{left:-35%;right:100%}60%{left:100%;right:-90%}to{left:100%;right:-90%}}@keyframes collectionsBlink{25%{opacity:.6}50%{opacity:.4}75%{opacity:.6}}\n"], dependencies: [{ kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "component", type: BbCollectionsViewer, selector: "[bb-collections-viewer]", inputs: ["collection"] }, { kind: "directive", type: BbCollectionsData, selector: "[bbCollectionsData]" }, { kind: "directive", type: BbCollectionsEmpty, selector: "[bbCollectionsEmpty]" }, { kind: "directive", type: BbCollectionsError, selector: "[bbCollectionsError]" }, { kind: "directive", type: BbCollectionsInitial, selector: "[bbCollectionsInitial]" }, { kind: "component", type: BbCollectionsActions, selector: "bb-collections-actions", inputs: ["collection", "queryPlaceholder", "standalone"] }, { kind: "component", type: BbCollectionsPager, selector: "bb-collections-pager", inputs: ["maxAmountOfButtons", "collection"] }, { kind: "component", type: BbLocalizeString, selector: "[bb-localize-string]", inputs: ["substitutions", "bb-localize-string"] }, { kind: "directive", type: BbTemplate, selector: "[bbTemplate]", inputs: ["bbTemplate"] }, { kind: "pipe", type: BbLocalize, name: "bbLocalize" }, { kind: "directive", type: BbLocalizeTemplate, selector: "[bbLocalizeTemplate]", inputs: ["bbLocalizeTemplate"] }, { kind: "component", type: BbSpinner, selector: "bb-spinner", inputs: ["color", "alt", "disabled"] }, { kind: "pipe", type: AsyncPipe, name: "async" }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsTable, decorators: [{
            type: Component,
            args: [{ selector: 'bb-collections-table', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, host: {
                        'class': 'bb-collections-table',
                        '[class.include-vertical-lines]': 'includeVerticalLines'
                    }, preserveWhitespaces: false, standalone: true, imports: [
                        NgTemplateOutlet,
                        BbCollectionsViewer,
                        BbCollectionsData,
                        BbCollectionsEmpty,
                        BbCollectionsError,
                        BbCollectionsInitial,
                        BbCollectionsActions,
                        BbCollectionsPager,
                        BbLocalizeString,
                        BbTemplate,
                        BbLocalize,
                        BbLocalizeTemplate,
                        BbSpinner,
                        AsyncPipe
                    ], template: "@if (!hideActions) {\n    <bb-collections-actions [collection]=\"collection\"\n                            [queryPlaceholder]=\"queryPlaceholder\">\n        <ng-content ngProjectAs=\"[data-collections-action]\"\n                    select=\"[data-collections-action]\"></ng-content>\n        <ng-content ngProjectAs=\"[data-collections-row]\"\n                    select=\"[data-collections-row]\"></ng-content>\n    </bb-collections-actions>\n}\n\n<div [class.loading]=\"collection?.loading$ | async\"\n     class=\"bb-collections-table-wrapper\">\n    <div [collection]=\"collection\"\n         class=\"bb-collections-table-container\"\n         bb-collections-viewer>\n        @if (collection?.loading$ | async) {\n            <div class=\"bb-collections-table-loading\"></div>\n        }\n        <table *bbCollectionsData=\"let items = items; let loading = loading\">\n            <ng-content select=\"thead\"></ng-content>\n            <tbody>\n                @for (item of items; track (trackByProperty === null ? $index : item[trackByProperty])) {\n                    <ng-container\n                        *ngTemplateOutlet=\"dataRowTemplate; context: {$implicit: item, item: item}\"></ng-container>\n                }\n            </tbody>\n            <ng-content select=\"tfoot\"></ng-content>\n        </table>\n        <div *bbCollectionsEmpty=\"let query = query\"\n             class=\"bb-collections-table-info\">\n            <div class=\"bb-collections-table-image empty\"></div>\n            @if (query?.length > 0) {\n                <ng-template [bbTemplate]=\"noResultsContent\">\n                    <p [bb-localize-string]=\"$any(noResultsContent) ?? ('collection.no_results_label' | bbLocalize)\"\n                       class=\"bb-collections-table-info-text\">\n                        <strong *bbLocalizeTemplate=\"'query'\">{{ query ?? '...' }}</strong>\n                    </p>\n                </ng-template>\n            } @else {\n                <ng-template [bbTemplate]=\"emptyContent\">\n                    <p class=\"bb-collections-table-info-text\">\n                        {{ $any(emptyContent) ?? ('collection.empty_label' | bbLocalize) }}\n                    </p>\n                </ng-template>\n            }\n        </div>\n        <div *bbCollectionsError=\"let error = error\"\n             class=\"bb-collections-table-info\">\n            <div class=\"bb-collections-table-image error\"></div>\n            <ng-template [bbTemplate]=\"errorContent\">\n                <p class=\"bb-collections-table-info-text\">\n                    {{ $any(errorContent) ?? ('collection.error_label' | bbLocalize) }}\n                </p>\n            </ng-template>\n        </div>\n        <div *bbCollectionsInitial\n             class=\"bb-collections-table-info\">\n            <bb-spinner class=\"block\"></bb-spinner>\n        </div>\n    </div>\n</div>\n\n@if (!hidePager) {\n    <footer class=\"bb-collections-table-footer\">\n        <bb-collections-pager [collection]=\"collection\"></bb-collections-pager>\n    </footer>\n}\n", styles: ["@charset \"UTF-8\";.bb-collections-table{color:#111;display:block;position:relative;font-size:.875rem;border-radius:.5rem;background-color:#fff;box-shadow:0 .375rem .375rem -.375rem #0000001a}.bb-collections-table>*:first-child{border-top-left-radius:.5rem;border-top-right-radius:.5rem}.bb-collections-table>*:last-child{border-bottom-left-radius:.5rem;border-bottom-right-radius:.5rem}.bb-collections-table.include-vertical-lines td:not(:last-child),.bb-collections-table.include-vertical-lines th:not(:last-child){border-right:1px solid hsl(213,8%,90%)}.bb-collections-table-footer{padding:.75rem 1rem;background-color:#fbf9f9;border-top:1px solid hsl(213,8%,90%)}.bb-collections-table-wrapper{overflow:hidden;position:relative}.bb-collections-table-wrapper.loading:after{top:0;left:0;right:0;z-index:1;opacity:0;content:\"\";display:block;height:.25rem;position:absolute;will-change:left,right;background-color:#1565c1;animation:collectionsFadeIn .12s cubic-bezier(.65,.815,.735,.395) .4s forwards,collectionsIndeterminate 2.1s cubic-bezier(.65,.815,.735,.395) .4s infinite}.bb-collections-table-container{overflow-x:auto;position:relative}.bb-collections-table-container.loading>table>tbody,.bb-collections-table-container.loading>.bb-collections-table-info{animation:collectionsBlink 1s linear infinite}.bb-collections-table-container>table{width:100%}.bb-collections-table-container>table th,.bb-collections-table-container>table td{padding:1rem;text-align:left;font-weight:400;font-size:.875rem}.bb-collections-table-container>table th.fit,.bb-collections-table-container>table td.fit{width:1%;white-space:nowrap}.bb-collections-table-container>table th.left,.bb-collections-table-container>table td.left{text-align:left}.bb-collections-table-container>table th.right,.bb-collections-table-container>table td.right{text-align:right}.bb-collections-table-container>table>thead{background-color:#fbf9f9;border-bottom:1px solid hsl(213,8%,90%)}.bb-collections-table-container>table>thead>tr>th{padding:1rem;color:#525252}.bb-collections-table-container>table>tbody>tr>td{color:#111}.bb-collections-table-container>table>tbody>tr:not(:last-child){border-bottom:1px solid hsl(213,8%,90%)}.bb-collections-table-info{padding:3rem 1.5rem}.bb-collections-table-info-text{color:#525252;margin:0 auto;display:block;line-height:2;max-width:30rem;text-align:center;font-size:.875rem}.bb-collections-table-info-text>strong{font-weight:500}.bb-collections-table-info-text>strong:before{content:\"\\201c\"}.bb-collections-table-info-text>strong:after{content:\"\\201d\"}.bb-collections-table-image{width:100%;height:auto;display:flex;max-width:7.5rem;margin:0 auto 1.5rem;background-size:contain;background-repeat:no-repeat;background-position:center center}.bb-collections-table-image.empty{aspect-ratio:154/132;mask-image:linear-gradient(to bottom,black 25%,transparent 100%);background-image:url('data:image/svg+xml,%3Csvg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 154 132\"%3E%3Cg fill=\"none\" fill-rule=\"evenodd\"%3E%3Crect width=\"154\" height=\"40\" fill=\"%23FFF\" fill-rule=\"nonzero\" rx=\"8\"/%3E%3Crect width=\"24\" height=\"24\" x=\"8\" y=\"8\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"4\"/%3E%3Crect width=\"60\" height=\"6\" x=\"40\" y=\"11\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"3\"/%3E%3Crect width=\"106\" height=\"6\" x=\"40\" y=\"23\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"3\"/%3E%3Crect width=\"153\" height=\"39\" x=\".5\" y=\".5\" stroke=\"%23E5E7EB\" rx=\"7.5\"/%3E%3Cg transform=\"translate(0 46)\"%3E%3Crect width=\"154\" height=\"40\" fill=\"%23FFF\" fill-rule=\"nonzero\" rx=\"8\"/%3E%3Crect width=\"24\" height=\"24\" x=\"8\" y=\"8\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"4\"/%3E%3Crect width=\"60\" height=\"6\" x=\"40\" y=\"11\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"3\"/%3E%3Crect width=\"106\" height=\"6\" x=\"40\" y=\"23\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"3\"/%3E%3Crect width=\"153\" height=\"39\" x=\".5\" y=\".5\" stroke=\"%23E5E7EB\" rx=\"7.5\"/%3E%3C/g%3E%3Cg transform=\"translate(0 92)\"%3E%3Crect width=\"154\" height=\"40\" fill=\"%23FFF\" fill-rule=\"nonzero\" rx=\"8\"/%3E%3Crect width=\"24\" height=\"24\" x=\"8\" y=\"8\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"4\"/%3E%3Crect width=\"60\" height=\"6\" x=\"40\" y=\"11\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"3\"/%3E%3Crect width=\"106\" height=\"6\" x=\"40\" y=\"23\" fill=\"%23E5E7EB\" fill-rule=\"nonzero\" rx=\"3\"/%3E%3Crect width=\"153\" height=\"39\" x=\".5\" y=\".5\" stroke=\"%23E5E7EB\" rx=\"7.5\"/%3E%3C/g%3E%3C/g%3E%3C/svg%3E')}.bb-collections-table-image.error{max-width:4rem;aspect-ratio:1;background-image:url('data:image/svg+xml,%3Csvg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 132 132\"%3E%3Cpath fill=\"%23C23934\" d=\"M65.958253 98.6024096c.8892771 0 1.6486747-.3008433 2.2781928-.9025301.6295181-.6016867.9442771-1.3471687.9442771-2.2364458 0-.8892771-.3008434-1.6486747-.9025301-2.2781927-.6016868-.6295181-1.3471687-.9442771-2.2364458-.9442771-.8892771 0-1.6486747.3008433-2.2781928.9025301-.6295181.6016867-.9442771 1.3471687-.9442771 2.2364458 0 .8892771.3008434 1.6486747.9025301 2.2781927.6016868.6295181 1.3471687.9442771 2.2364458.9442771Zm-1.9462048-21.4698795h3.9759036V29.4216867h-3.9759036v47.7108434ZM66.0894578 132c-9.0716867 0-17.6205421-1.715602-25.6465662-5.146807-8.0260241-3.431205-15.0547591-8.159217-21.0862049-14.184036-6.032771-6.02482-10.76674694-13.0416269-14.20192766-21.0504221C1.71825301 83.6086145 0 75.0504819 0 65.9443373 0 56.8368675 1.71560241 48.303253 5.14680723 40.343494c3.43120482-7.9597591 8.15921687-14.9553615 14.18403617-20.9868073 6.0248193-6.032771 13.0416265-10.76674694 21.0504217-14.20192766C48.3913855 1.71825301 56.9495181 0 66.0556627 0 75.1631325 0 83.696747 1.71560241 91.656506 5.14680723c7.9597591 3.43120482 14.955361 8.15921687 20.986807 14.18403617 6.032771 6.0248193 10.766747 13.0263855 14.201928 21.0046988C130.281747 48.3138554 132 56.8388554 132 65.9105422c0 9.0716867-1.715602 17.6205421-5.146807 25.6465662-3.431205 8.0260241-8.159217 15.0547586-14.184036 21.0862046-6.02482 6.032771-13.0263859 10.766747-21.0046992 14.201928C83.6861446 130.281747 75.1611446 132 66.0894578 132ZM66 128.024096c17.2289157 0 31.873494-6.03012 43.933735-18.090361C121.993976 97.873494 128.024096 83.2289157 128.024096 66c0-17.2289157-6.03012-31.873494-18.090361-43.9337349C97.873494 10.0060241 83.2289157 3.97590361 66 3.97590361c-17.2289157 0-31.873494 6.03012049-43.9337349 18.09036149C10.0060241 34.126506 3.97590361 48.7710843 3.97590361 66c0 17.2289157 6.03012049 31.873494 18.09036149 43.933735C34.126506 121.993976 48.7710843 128.024096 66 128.024096Z\"/%3E%3C/svg%3E')}@keyframes collectionsFadeIn{0%{opacity:0}to{opacity:1}}@keyframes collectionsIndeterminate{0%{left:-35%;right:100%}60%{left:100%;right:-90%}to{left:100%;right:-90%}}@keyframes collectionsBlink{25%{opacity:.6}50%{opacity:.4}75%{opacity:.6}}\n"] }]
        }], propDecorators: { dataRowTemplate: [{
                type: ContentChild,
                args: [BbCollectionsDataRow, { read: TemplateRef, static: false }]
            }], queryPlaceholder: [{
                type: Input
            }], emptyContent: [{
                type: Input
            }], noResultsContent: [{
                type: Input
            }], errorContent: [{
                type: Input
            }], trackByProperty: [{
                type: Input
            }], includeVerticalLines: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], hideActions: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], hidePager: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], collection: [{
                type: Input,
                args: [{ required: true }]
            }] } });

class BbCollectionsTableSorting {
    _name;
    _changeDetectorRef;
    _parent;
    // Readonly data.
    orderedOptions = [null, 'asc', 'desc'];
    // State.
    direction = null;
    // Subscription.
    _subscription = new Subscription();
    constructor(_name, _changeDetectorRef, _parent) {
        this._name = _name;
        this._changeDetectorRef = _changeDetectorRef;
        this._parent = _parent;
    }
    ngOnInit() {
        const direction$ = this._parent.collection$.pipe(switchMap(collection => getControlValue(collection.sortControl)), map(value => Array.isArray(value) ? value : [value]), map(value => value?.find(item => item?.name === this._name)), map(sorting => sorting?.direction ?? null), distinctUntilChanged());
        const subscription = direction$.subscribe(direction => {
            this.direction = direction;
            this._changeDetectorRef.markForCheck();
        });
        this._subscription.add(subscription);
    }
    ngOnDestroy() {
        this._subscription?.unsubscribe();
    }
    onClick() {
        if (!this._name) {
            return;
        }
        const nextDirection = this.getNextOption(this.direction);
        const controlValue = nextDirection === null ? null : { name: this._name, direction: nextDirection };
        this._parent?.collection?.sortControl?.setValue(controlValue);
    }
    getNextOption(direction) {
        const index = this.orderedOptions.findIndex(item => item === direction);
        const newIndex = (index + 1) % this.orderedOptions?.length;
        return this.orderedOptions?.[newIndex] ?? null;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsTableSorting, deps: [{ token: 'bb-collections-table-sorting', attribute: true }, { token: i0.ChangeDetectorRef }, { token: BbCollectionsTable, optional: true }], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.8", type: BbCollectionsTableSorting, isStandalone: true, selector: "th[bb-collections-table-sorting]", host: { listeners: { "click": "onClick()" }, properties: { "class.asc": "direction === 'asc'", "class.desc": "direction === 'desc'" }, classAttribute: "bb-collections-table-sorting" }, ngImport: i0, template: "<ng-content></ng-content>\n", styles: [".bb-collections-table-sorting{cursor:pointer;-webkit-user-select:none;user-select:none;background-image:url('data:image/svg+xml,%3Csvg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 9 15\" width=\"9\" height=\"15\"%3E%3Cpath fill=\"%23dcdcdc\" d=\"M4.5 0L0 7h9z\"%3E%3C/path%3E%3Cpath fill=\"%23dcdcdc\" d=\"M4.5 15L9 8H0z\"%3E%3C/path%3E%3C/svg%3E');background-repeat:no-repeat;padding-right:2.5rem!important;background-size:.5625rem .9375rem;background-position:center right 1rem;transition:background-color .12s cubic-bezier(0,0,.2,1)}.bb-collections-table-sorting:hover,.bb-collections-table-sorting:focus{background-color:#f4f0f0}.bb-collections-table-sorting:active{background-color:#f0ebeb}.bb-collections-table-sorting.asc{background-image:url('data:image/svg+xml,%3Csvg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 9 15\" width=\"9\" height=\"15\"%3E%3Cpath fill=\"black\" d=\"M4.5 0L0 7h9z\"%3E%3C/path%3E%3Cpath fill=\"%23dcdcdc\" d=\"M4.5 15L9 8H0z\"%3E%3C/path%3E%3C/svg%3E')}.bb-collections-table-sorting.desc{background-image:url('data:image/svg+xml,%3Csvg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 9 15\" width=\"9\" height=\"15\"%3E%3Cpath fill=\"%23dcdcdc\" d=\"M4.5 0L0 7h9z\"%3E%3C/path%3E%3Cpath fill=\"black\" d=\"M4.5 15L9 8H0z\"%3E%3C/path%3E%3C/svg%3E')}\n"], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsTableSorting, decorators: [{
            type: Component,
            args: [{ selector: 'th[bb-collections-table-sorting]', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, host: {
                        'class': 'bb-collections-table-sorting',
                        '[class.asc]': 'direction === \'asc\'',
                        '[class.desc]': 'direction === \'desc\''
                    }, preserveWhitespaces: false, standalone: true, template: "<ng-content></ng-content>\n", styles: [".bb-collections-table-sorting{cursor:pointer;-webkit-user-select:none;user-select:none;background-image:url('data:image/svg+xml,%3Csvg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 9 15\" width=\"9\" height=\"15\"%3E%3Cpath fill=\"%23dcdcdc\" d=\"M4.5 0L0 7h9z\"%3E%3C/path%3E%3Cpath fill=\"%23dcdcdc\" d=\"M4.5 15L9 8H0z\"%3E%3C/path%3E%3C/svg%3E');background-repeat:no-repeat;padding-right:2.5rem!important;background-size:.5625rem .9375rem;background-position:center right 1rem;transition:background-color .12s cubic-bezier(0,0,.2,1)}.bb-collections-table-sorting:hover,.bb-collections-table-sorting:focus{background-color:#f4f0f0}.bb-collections-table-sorting:active{background-color:#f0ebeb}.bb-collections-table-sorting.asc{background-image:url('data:image/svg+xml,%3Csvg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 9 15\" width=\"9\" height=\"15\"%3E%3Cpath fill=\"black\" d=\"M4.5 0L0 7h9z\"%3E%3C/path%3E%3Cpath fill=\"%23dcdcdc\" d=\"M4.5 15L9 8H0z\"%3E%3C/path%3E%3C/svg%3E')}.bb-collections-table-sorting.desc{background-image:url('data:image/svg+xml,%3Csvg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 9 15\" width=\"9\" height=\"15\"%3E%3Cpath fill=\"%23dcdcdc\" d=\"M4.5 0L0 7h9z\"%3E%3C/path%3E%3Cpath fill=\"black\" d=\"M4.5 15L9 8H0z\"%3E%3C/path%3E%3C/svg%3E')}\n"] }]
        }], ctorParameters: () => [{ type: undefined, decorators: [{
                    type: Attribute,
                    args: ['bb-collections-table-sorting']
                }] }, { type: i0.ChangeDetectorRef }, { type: BbCollectionsTable, decorators: [{
                    type: Optional
                }] }], propDecorators: { onClick: [{
                type: HostListener,
                args: ['click']
            }] } });

class BbCollectionsTableLabel {
    // Inputs.
    titleLabel = null;
    descriptionLabel = null;
    imageUrl = null;
    alt = null;
    type = 'image';
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsTableLabel, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "18.2.8", type: BbCollectionsTableLabel, isStandalone: true, selector: "[bb-collections-table-label]", inputs: { titleLabel: "titleLabel", descriptionLabel: "descriptionLabel", imageUrl: "imageUrl", alt: "alt", type: "type" }, host: { classAttribute: "bb-collections-table-label" }, ngImport: i0, template: "@switch (type) {\n    @case ('avatar') {\n        <bb-avatar [title]=\"alt ?? '...'\"\n                   [src]=\"imageUrl\"\n                   size=\"2\"\n                   class=\"bb-collections-table-label-avatar\">\n        </bb-avatar>\n    }\n    @case ('content') {\n        <ng-content></ng-content>\n    }\n    @default {\n        @if (imageUrl; as src) {\n            <div [style.background-image]=\"'url(' + src + ')'\"\n                 class=\"bb-collections-table-label-image\"></div>\n        }\n    }\n}\n\n<div class=\"bb-collections-table-label-content\">\n    <span class=\"bb-collections-table-label-title\"><ng-template\n        [bbTemplate]=\"titleLabel\">{{ titleLabel }}</ng-template></span>\n    @if (descriptionLabel; as descriptionContent) {\n        <small class=\"bb-collections-table-label-description\">\n            <ng-template [bbTemplate]=\"descriptionContent\">{{ descriptionContent }}</ng-template>\n        </small>\n    }\n</div>\n", styles: [".bb-collections-table-label{align-items:center;display:inline-flex}a.bb-collections-table-label{text-decoration:none}a.bb-collections-table-label>.bb-collections-table-label-content>.bb-collections-table-label-title,a.bb-collections-table-label:visited>.bb-collections-table-label-content>.bb-collections-table-label-title{text-underline-offset:.25rem;text-decoration:dotted underline;text-decoration-thickness:1px;color:var(--bb-collections-table-label-color)}a.bb-collections-table-label>.bb-collections-table-label-content>.bb-collections-table-label-description,a.bb-collections-table-label:visited>.bb-collections-table-label-content>.bb-collections-table-label-description{color:#758795}a.bb-collections-table-label:hover>.bb-collections-table-label-content>.bb-collections-table-label-title,a.bb-collections-table-label:focus>.bb-collections-table-label-content>.bb-collections-table-label-title{text-decoration-style:solid}.bb-collections-table-label-image,.bb-collections-table-label-avatar{width:2rem;height:2rem;min-width:2rem;min-height:2rem;margin-right:.5rem}.bb-collections-table-label-image{background-size:contain;background-repeat:no-repeat;background-position:center center}.bb-collections-table-label-content{display:block}.bb-collections-table-label-title{display:block;color:#111;line-height:1.6;font-size:.875rem}.bb-collections-table-label-description{display:block;line-height:1.6;margin-top:.25rem;font-size:.8125rem;color:#758795}\n"], dependencies: [{ kind: "component", type: BbAvatar, selector: "bb-avatar", inputs: ["size", "unit", "src", "title", "tintColor"] }, { kind: "directive", type: BbTemplate, selector: "[bbTemplate]", inputs: ["bbTemplate"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsTableLabel, decorators: [{
            type: Component,
            args: [{ selector: '[bb-collections-table-label]', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, host: { 'class': 'bb-collections-table-label' }, preserveWhitespaces: false, standalone: true, imports: [BbAvatar, BbTemplate], template: "@switch (type) {\n    @case ('avatar') {\n        <bb-avatar [title]=\"alt ?? '...'\"\n                   [src]=\"imageUrl\"\n                   size=\"2\"\n                   class=\"bb-collections-table-label-avatar\">\n        </bb-avatar>\n    }\n    @case ('content') {\n        <ng-content></ng-content>\n    }\n    @default {\n        @if (imageUrl; as src) {\n            <div [style.background-image]=\"'url(' + src + ')'\"\n                 class=\"bb-collections-table-label-image\"></div>\n        }\n    }\n}\n\n<div class=\"bb-collections-table-label-content\">\n    <span class=\"bb-collections-table-label-title\"><ng-template\n        [bbTemplate]=\"titleLabel\">{{ titleLabel }}</ng-template></span>\n    @if (descriptionLabel; as descriptionContent) {\n        <small class=\"bb-collections-table-label-description\">\n            <ng-template [bbTemplate]=\"descriptionContent\">{{ descriptionContent }}</ng-template>\n        </small>\n    }\n</div>\n", styles: [".bb-collections-table-label{align-items:center;display:inline-flex}a.bb-collections-table-label{text-decoration:none}a.bb-collections-table-label>.bb-collections-table-label-content>.bb-collections-table-label-title,a.bb-collections-table-label:visited>.bb-collections-table-label-content>.bb-collections-table-label-title{text-underline-offset:.25rem;text-decoration:dotted underline;text-decoration-thickness:1px;color:var(--bb-collections-table-label-color)}a.bb-collections-table-label>.bb-collections-table-label-content>.bb-collections-table-label-description,a.bb-collections-table-label:visited>.bb-collections-table-label-content>.bb-collections-table-label-description{color:#758795}a.bb-collections-table-label:hover>.bb-collections-table-label-content>.bb-collections-table-label-title,a.bb-collections-table-label:focus>.bb-collections-table-label-content>.bb-collections-table-label-title{text-decoration-style:solid}.bb-collections-table-label-image,.bb-collections-table-label-avatar{width:2rem;height:2rem;min-width:2rem;min-height:2rem;margin-right:.5rem}.bb-collections-table-label-image{background-size:contain;background-repeat:no-repeat;background-position:center center}.bb-collections-table-label-content{display:block}.bb-collections-table-label-title{display:block;color:#111;line-height:1.6;font-size:.875rem}.bb-collections-table-label-description{display:block;line-height:1.6;margin-top:.25rem;font-size:.8125rem;color:#758795}\n"] }]
        }], propDecorators: { titleLabel: [{
                type: Input,
                args: [{ required: true }]
            }], descriptionLabel: [{
                type: Input
            }], imageUrl: [{
                type: Input
            }], alt: [{
                type: Input
            }], type: [{
                type: Input
            }] } });

class BbCollectionsGrid {
    // Readonly data.
    collection$ = new BehaviorSubject(null);
    // Views.
    itemTemplate;
    // Inputs.
    queryPlaceholder = null;
    hideActions = false;
    hidePager = false;
    set collection(collection) {
        this.collection$.next(collection);
    }
    get collection() {
        return this.collection$.getValue();
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsGrid, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "18.2.8", type: BbCollectionsGrid, isStandalone: true, selector: "bb-collections-grid", inputs: { queryPlaceholder: "queryPlaceholder", hideActions: ["hideActions", "hideActions", booleanAttribute], hidePager: ["hidePager", "hidePager", booleanAttribute], collection: "collection" }, host: { classAttribute: "bb-collections-grid" }, queries: [{ propertyName: "itemTemplate", first: true, predicate: BbCollectionsItem, descendants: true, read: TemplateRef }], ngImport: i0, template: "@if (!hideActions) {\n    <bb-collections-actions [collection]=\"collection\"\n                            [queryPlaceholder]=\"queryPlaceholder\"\n                            standalone>\n        <ng-content ngProjectAs=\"[data-collections-action]\"\n                    select=\"[data-collections-action]\"></ng-content>\n        <ng-content ngProjectAs=\"[data-collections-row]\"\n                    select=\"[data-collections-row]\"></ng-content>\n    </bb-collections-actions>\n}\n\n<div [collection]=\"collection\"\n     class=\"bb-collections-grid-container\"\n     bb-collections-viewer>\n    <ul *bbCollectionsData=\"let items = items\"\n        class=\"bb-collections-grid-list\">\n        @for (item of items; track $index) {\n            <li class=\"bb-collections-grid-item\">\n                <ng-container *ngTemplateOutlet=\"itemTemplate; context: {$implicit: item, item: item}\"></ng-container>\n            </li>\n        }\n    </ul>\n</div>\n\n@if (!hidePager) {\n    <footer class=\"bb-collections-grid-footer\">\n        <bb-collections-pager [collection]=\"collection\"></bb-collections-pager>\n    </footer>\n}\n", styles: [".bb-collections-grid{display:block}.bb-collections-grid-list{gap:1.5rem;display:flex;flex-wrap:wrap}.bb-collections-grid-item{width:100%;display:flex}.bb-collections-grid-footer{margin-top:1.5rem}\n"], dependencies: [{ kind: "directive", type: BbCollectionsData, selector: "[bbCollectionsData]" }, { kind: "component", type: BbCollectionsViewer, selector: "[bb-collections-viewer]", inputs: ["collection"] }, { kind: "directive", type: NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "component", type: BbCollectionsActions, selector: "bb-collections-actions", inputs: ["collection", "queryPlaceholder", "standalone"] }, { kind: "component", type: BbCollectionsPager, selector: "bb-collections-pager", inputs: ["maxAmountOfButtons", "collection"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbCollectionsGrid, decorators: [{
            type: Component,
            args: [{ selector: 'bb-collections-grid', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, host: { 'class': 'bb-collections-grid' }, preserveWhitespaces: false, standalone: true, imports: [
                        BbCollectionsData,
                        BbCollectionsViewer,
                        NgTemplateOutlet,
                        BbCollectionsActions,
                        BbCollectionsPager
                    ], template: "@if (!hideActions) {\n    <bb-collections-actions [collection]=\"collection\"\n                            [queryPlaceholder]=\"queryPlaceholder\"\n                            standalone>\n        <ng-content ngProjectAs=\"[data-collections-action]\"\n                    select=\"[data-collections-action]\"></ng-content>\n        <ng-content ngProjectAs=\"[data-collections-row]\"\n                    select=\"[data-collections-row]\"></ng-content>\n    </bb-collections-actions>\n}\n\n<div [collection]=\"collection\"\n     class=\"bb-collections-grid-container\"\n     bb-collections-viewer>\n    <ul *bbCollectionsData=\"let items = items\"\n        class=\"bb-collections-grid-list\">\n        @for (item of items; track $index) {\n            <li class=\"bb-collections-grid-item\">\n                <ng-container *ngTemplateOutlet=\"itemTemplate; context: {$implicit: item, item: item}\"></ng-container>\n            </li>\n        }\n    </ul>\n</div>\n\n@if (!hidePager) {\n    <footer class=\"bb-collections-grid-footer\">\n        <bb-collections-pager [collection]=\"collection\"></bb-collections-pager>\n    </footer>\n}\n", styles: [".bb-collections-grid{display:block}.bb-collections-grid-list{gap:1.5rem;display:flex;flex-wrap:wrap}.bb-collections-grid-item{width:100%;display:flex}.bb-collections-grid-footer{margin-top:1.5rem}\n"] }]
        }], propDecorators: { itemTemplate: [{
                type: ContentChild,
                args: [BbCollectionsItem, { read: TemplateRef, static: false }]
            }], queryPlaceholder: [{
                type: Input
            }], hideActions: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], hidePager: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], collection: [{
                type: Input,
                args: [{ required: true }]
            }] } });

class Collection {
    _provider;
    _initialValues;
    // Controls.
    queryControl;
    queryFieldsControl;
    pageNumberControl;
    pageSizeControl;
    sortControl;
    // Request.
    data$;
    raw$ = new BehaviorSubject(null);
    loading$ = new BehaviorSubject(true);
    // State.
    _formatter = null;
    _refresh$ = new BehaviorSubject(0);
    _requestCache = null;
    constructor(_provider, _initialValues = {}) {
        this._provider = _provider;
        this._initialValues = _initialValues;
        this.queryControl = new FormControl(_initialValues?.query ?? null);
        this.queryFieldsControl = new FormControl(_initialValues?.queryFields ?? []);
        this.pageNumberControl = new FormControl(_initialValues?.pageNumber ?? 1);
        this.pageSizeControl = new FormControl(_initialValues?.pageSize ?? 25);
        this.sortControl = new FormControl(_initialValues?.sort ?? null);
        this.data$ = this.getData();
    }
    refresh() {
        this._refresh$.next(0);
    }
    setFormatter(callback) {
        this._formatter = callback;
        return this;
    }
    getData() {
        const data$ = this.request.pipe(tap(() => this.loading$.next(true)), map(request => this.validateRequest(request)), switchMap(request => {
            return this._provider.transform(request).pipe(tap(response => this.cacheCollectionData(response)), map(response => ({ response, request, initial: false })), catchError(error => of({
                error,
                request,
                response: null,
                initial: false
            })), tap(() => this.loading$.next(false)));
        }), startWith({ initial: true, response: null }), shareReplay({ refCount: true, bufferSize: 1 }));
        return combineLatest([data$, this.loading$.asObservable()]).pipe(switchMap(([data, loading]) => {
            const combined = { ...data, loading };
            return this.formatResponse(data?.response).pipe(map(response => {
                const data = { ...combined, response };
                return { ...data, state: this.getState(data) };
            }));
        }));
    }
    formatResponse(response) {
        if (!response || !this._formatter) {
            return of(response);
        }
        const items = response?.data ?? [];
        const data = this._formatter?.(items) ?? items;
        const data$ = isObservable(data) ? data : of(data);
        return data$.pipe(map(data => ({ ...response, data })));
    }
    validateRequest(request) {
        // We should reset the page number whenever we change the query or the page size.
        if (this._requestCache && request && (this._requestCache?.query !== request?.query || this._requestCache?.pageSize !== request?.pageSize)) {
            this.pageNumberControl?.setValue(1, { emitEvent: false });
            request.pageNumber = this.pageNumberControl?.value;
        }
        this._requestCache = { ...request };
        return request;
    }
    getState(data) {
        if (!!data?.error) {
            return "error" /* CollectionState.Error */;
        }
        if (!!data?.initial) {
            return "initial" /* CollectionState.Initial */;
        }
        const count = data?.response?.data?.length ?? 0;
        if (count <= 0) {
            return "empty" /* CollectionState.Empty */;
        }
        return "data" /* CollectionState.Data */;
    }
    cacheCollectionData(data) {
        this.raw$.next(data);
    }
    get request() {
        const pageNumber$ = getControlValue(this.pageNumberControl);
        const pageSize$ = getControlValue(this.pageSizeControl);
        const query$ = this.getQueryObservable();
        const queryFields$ = getControlValue(this.queryFieldsControl);
        const sort$ = getControlValue(this.sortControl);
        return combineLatest([pageNumber$, pageSize$, query$, queryFields$, sort$, this._refresh$.asObservable()]).pipe(map(([pageNumber, pageSize, query, queryFields, sort]) => ({
            pageNumber,
            pageSize,
            query,
            queryFields,
            sort
        })));
    }
    getQueryObservable() {
        return this.queryControl.valueChanges.pipe(debounceTime(400), distinctUntilChanged(), startWith(this.queryControl?.value));
    }
}

class CollectionsModule {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: CollectionsModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
    static ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "18.2.8", ngImport: i0, type: CollectionsModule, imports: [BbCollectionsViewer,
            BbCollectionsPager,
            BbCollectionsTable,
            BbCollectionsTableSorting,
            BbCollectionsTableLabel,
            BbCollectionsGrid,
            BbCollectionsActions,
            BbCollectionsData,
            BbCollectionsDataRow,
            BbCollectionsEmpty,
            BbCollectionsError,
            BbCollectionsInitial,
            BbCollectionsItem], exports: [BbCollectionsViewer,
            BbCollectionsPager,
            BbCollectionsTable,
            BbCollectionsTableSorting,
            BbCollectionsTableLabel,
            BbCollectionsGrid,
            BbCollectionsActions,
            BbCollectionsData,
            BbCollectionsDataRow,
            BbCollectionsEmpty,
            BbCollectionsError,
            BbCollectionsInitial,
            BbCollectionsItem] });
    static ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: CollectionsModule, imports: [BbCollectionsPager,
            BbCollectionsTable,
            BbCollectionsTableLabel,
            BbCollectionsGrid,
            BbCollectionsActions] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: CollectionsModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [
                        BbCollectionsViewer,
                        BbCollectionsPager,
                        BbCollectionsTable,
                        BbCollectionsTableSorting,
                        BbCollectionsTableLabel,
                        BbCollectionsGrid,
                        BbCollectionsActions,
                        BbCollectionsData,
                        BbCollectionsDataRow,
                        BbCollectionsEmpty,
                        BbCollectionsError,
                        BbCollectionsInitial,
                        BbCollectionsItem
                    ],
                    exports: [
                        BbCollectionsViewer,
                        BbCollectionsPager,
                        BbCollectionsTable,
                        BbCollectionsTableSorting,
                        BbCollectionsTableLabel,
                        BbCollectionsGrid,
                        BbCollectionsActions,
                        BbCollectionsData,
                        BbCollectionsDataRow,
                        BbCollectionsEmpty,
                        BbCollectionsError,
                        BbCollectionsInitial,
                        BbCollectionsItem
                    ]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { ApiCollectionProvider, ArrayCollection, BbCollectionsActions, BbCollectionsData, BbCollectionsDataContext, BbCollectionsDataRow, BbCollectionsDataRowContext, BbCollectionsEmpty, BbCollectionsEmptyContext, BbCollectionsError, BbCollectionsErrorContext, BbCollectionsGrid, BbCollectionsInitial, BbCollectionsInitialContext, BbCollectionsItem, BbCollectionsItemContext, BbCollectionsPager, BbCollectionsTable, BbCollectionsTableLabel, BbCollectionsTableSorting, BbCollectionsViewer, Collection, CollectionProvider, CollectionsModule, LocalCollectionProvider };
//# sourceMappingURL=bravobit-bb-foundation-collections.mjs.map
