import { take, switchMap } from 'rxjs/operators';
import { Subject, Subscription, lastValueFrom, fromEvent, race, firstValueFrom } from 'rxjs';
import * as i0 from '@angular/core';
import { InjectionToken, inject, ChangeDetectorRef, EventEmitter, Component, ChangeDetectionStrategy, ViewEncapsulation, ViewChild, Output, HostListener, booleanAttribute, Optional, Inject, HostBinding, Input, Directive, createEnvironmentInjector, createComponent, Injectable, makeEnvironmentProviders, NgModule } from '@angular/core';
import { trigger, transition, group, query, style, animate } from '@angular/animations';
import { DOCUMENT, NgClass } from '@angular/common';
import * as i1 from '@bravobit/bb-foundation';
import { BbFormGroup, BbButton } from '@bravobit/bb-foundation/elements';
import { BbFocusTrap, BbTemplate } from '@bravobit/bb-foundation/utils';
import { BbLocalize } from '@bravobit/bb-foundation/localize';

class BbDialogRef {
    // Data.
    _afterClosed$ = new Subject();
    _overlayClicked$ = new Subject();
    get afterClosed() {
        return this._afterClosed$.pipe(take(1));
    }
    get overlayClicked() {
        return this._overlayClicked$.pipe(take(1));
    }
    onOverlayClicked() {
        this._overlayClicked$.next();
    }
    close(result) {
        this._afterClosed$.next(result);
    }
}

class BbDialogConfig {
    data;
}
class DialogConfig {
    disableContainerAnimations;
    disableOverlayAnimations;
}
const DIALOG_CONFIG = new InjectionToken('dialog config');

const bbDialogOverlayAnimation = trigger('bbDialogOverlayAnimation', [
    transition(':enter, * => open', [
        group([
            query('.bb-dialog-overlay-backdrop', [
                style({ opacity: 0 }),
                animate('240ms cubic-bezier(0, 0, 0.2, 1)', style({ opacity: 1 }))
            ]),
            query('.bb-dialog-modal', [
                style({ opacity: 0, transform: 'scale(0.98)' }),
                animate('200ms cubic-bezier(0, 0, 0.2, 1)', style({ opacity: 1, transform: 'none' }))
            ])
        ])
    ]),
    transition(':leave, * => close', [
        group([
            query('.bb-dialog-modal', [
                animate('200ms cubic-bezier(0, 0, 0.2, 1)', style({ opacity: 0, transform: 'scale(0.98)' }))
            ]),
            query('.bb-dialog-overlay-backdrop', [
                animate('240ms cubic-bezier(0, 0, 0.2, 1)', style({ opacity: 0 }))
            ])
        ])
    ])
]);

class BbDialogOverlay {
    // Dependencies.
    _config = inject(DIALOG_CONFIG, { optional: true });
    _changeDetectorRef = inject(ChangeDetectorRef);
    // Readonly data.
    disableOverlayAnimations = this._config?.disableOverlayAnimations ?? false;
    // Views.
    overlayElementRef;
    // Outputs.
    closeRequested = new EventEmitter();
    // State.
    state = 'open';
    _closed$ = new Subject();
    // Subscriptions.
    _subscription = new Subscription();
    ngOnInit() {
        this.listenForClicks();
    }
    ngOnDestroy() {
        this._subscription?.unsubscribe();
    }
    onKeyEvent(event) {
        if (event?.key === 'Escape') {
            this.closeRequested.emit();
        }
    }
    onAnimationDone(event) {
        // Check if the dialog is being destroyed.
        if (!['close'].includes(event?.toState)) {
            return;
        }
        this._closed$.next();
        this._closed$.complete();
    }
    close() {
        // We need change detection to run
        // here else it won't close in all scenario's.
        this.state = 'close';
        this._changeDetectorRef.markForCheck();
        // Listen for the closed trigger.
        const observable$ = this._closed$.pipe(take(1));
        return lastValueFrom(observable$);
    }
    listenForClicks() {
        const element = this.overlayElementRef?.nativeElement ?? null;
        if (!element) {
            return;
        }
        const mouseDown$ = fromEvent(element, 'mousedown');
        const touchStart$ = fromEvent(element, 'touchstart');
        const start$ = race([mouseDown$, touchStart$]);
        const mouseUp$ = fromEvent(element, 'mouseup');
        const touchEnd$ = fromEvent(element, 'touchend');
        const end$ = race([mouseUp$, touchEnd$]);
        const clicks$ = start$.pipe(switchMap(() => end$));
        const subscription = clicks$.subscribe(() => this.closeRequested.next());
        this._subscription.add(subscription);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogOverlay, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.8", type: BbDialogOverlay, isStandalone: true, selector: "bb-dialog-overlay", outputs: { closeRequested: "closeRequested" }, host: { listeners: { "window:keyup": "onKeyEvent($event)", "@bbDialogOverlayAnimation.done": "onAnimationDone($event)" }, properties: { "@bbDialogOverlayAnimation": "state", "@.disabled": "disableOverlayAnimations" }, classAttribute: "bb-dialog-overlay" }, viewQueries: [{ propertyName: "overlayElementRef", first: true, predicate: ["overlay"], descendants: true, static: true }], ngImport: i0, template: "<div #overlay\n     class=\"bb-dialog-overlay-backdrop\"></div>\n\n<ng-content select=\"[bb-dialog-modal]\"></ng-content>\n", styles: [".bb-dialog-overlay,.bb-dialog-overlay-backdrop{inset:0;display:flex;z-index:1000;position:fixed;flex-direction:column}.bb-dialog-overlay{padding:1.5rem}.bb-dialog-overlay-backdrop{will-change:opacity;background-color:#1119}.bb-dialog-overlay-component{z-index:1001;display:block;position:relative}\n"], animations: [bbDialogOverlayAnimation], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogOverlay, decorators: [{
            type: Component,
            args: [{ selector: 'bb-dialog-overlay', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, host: {
                        'class': 'bb-dialog-overlay',
                        '[@bbDialogOverlayAnimation]': 'state',
                        '[@.disabled]': 'disableOverlayAnimations'
                    }, preserveWhitespaces: false, animations: [bbDialogOverlayAnimation], standalone: true, template: "<div #overlay\n     class=\"bb-dialog-overlay-backdrop\"></div>\n\n<ng-content select=\"[bb-dialog-modal]\"></ng-content>\n", styles: [".bb-dialog-overlay,.bb-dialog-overlay-backdrop{inset:0;display:flex;z-index:1000;position:fixed;flex-direction:column}.bb-dialog-overlay{padding:1.5rem}.bb-dialog-overlay-backdrop{will-change:opacity;background-color:#1119}.bb-dialog-overlay-component{z-index:1001;display:block;position:relative}\n"] }]
        }], propDecorators: { overlayElementRef: [{
                type: ViewChild,
                args: ['overlay', { static: true }]
            }], closeRequested: [{
                type: Output
            }], onKeyEvent: [{
                type: HostListener,
                args: ['window:keyup', ['$event']]
            }], onAnimationDone: [{
                type: HostListener,
                args: ['@bbDialogOverlayAnimation.done', ['$event']]
            }] } });

class BbDialogModal {
    _patch;
    _renderer;
    _document;
    // Styling.
    maxWidth = '30rem';
    withoutBodyPadding = false;
    constructor(_patch, _renderer, _document) {
        this._patch = _patch;
        this._renderer = _renderer;
        this._document = _document;
    }
    ngOnInit() {
        this._patch.mobileVerticalHeight();
        this.allowGlobalOverflow(false);
    }
    ngOnDestroy() {
        this.allowGlobalOverflow(true);
    }
    allowGlobalOverflow(allow) {
        const element = this._document?.body ?? null;
        if (!element) {
            return;
        }
        allow
            ? this._renderer.removeStyle(element, 'overflow')
            : this._renderer.setStyle(element, 'overflow', 'hidden');
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogModal, deps: [{ token: i1.Patch }, { token: i0.Renderer2 }, { token: DOCUMENT, optional: true }], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "16.1.0", version: "18.2.8", type: BbDialogModal, isStandalone: true, selector: "[bb-dialog-modal]", inputs: { maxWidth: "maxWidth", withoutBodyPadding: ["withoutBodyPadding", "withoutBodyPadding", booleanAttribute] }, host: { attributes: { "role": "dialog" }, properties: { "class.without-body-padding": "withoutBodyPadding", "style.max-width": "this.maxWidth" }, classAttribute: "bb-dialog-modal" }, ngImport: i0, template: "<ng-content select=\"[bb-dialog-header]\"></ng-content>\n\n<div class=\"bb-dialog-modal-body\">\n    <ng-content></ng-content>\n</div>\n\n<ng-content select=\"[bb-dialog-actions]\"></ng-content>\n", styles: [".bb-dialog-modal{width:100%;margin:auto;z-index:1001;display:flex;max-width:25rem;position:relative;pointer-events:auto;border-radius:.25rem;flex-direction:column;background-color:#fff;will-change:opacity,transform;max-height:calc(var(--vh, 1vh) * 100 - 3rem)}.bb-dialog-modal.without-body-padding>.bb-dialog-modal-body{padding:0}.bb-dialog-modal-body{color:#738694;padding:1.5rem;overflow-y:auto;line-height:1.4;font-size:.8125rem;background-color:#f1f3f6}\n"], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogModal, decorators: [{
            type: Component,
            args: [{ selector: '[bb-dialog-modal]', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, host: {
                        'class': 'bb-dialog-modal',
                        '[class.without-body-padding]': 'withoutBodyPadding',
                        'role': 'dialog'
                    }, preserveWhitespaces: false, standalone: true, template: "<ng-content select=\"[bb-dialog-header]\"></ng-content>\n\n<div class=\"bb-dialog-modal-body\">\n    <ng-content></ng-content>\n</div>\n\n<ng-content select=\"[bb-dialog-actions]\"></ng-content>\n", styles: [".bb-dialog-modal{width:100%;margin:auto;z-index:1001;display:flex;max-width:25rem;position:relative;pointer-events:auto;border-radius:.25rem;flex-direction:column;background-color:#fff;will-change:opacity,transform;max-height:calc(var(--vh, 1vh) * 100 - 3rem)}.bb-dialog-modal.without-body-padding>.bb-dialog-modal-body{padding:0}.bb-dialog-modal-body{color:#738694;padding:1.5rem;overflow-y:auto;line-height:1.4;font-size:.8125rem;background-color:#f1f3f6}\n"] }]
        }], ctorParameters: () => [{ type: i1.Patch }, { type: i0.Renderer2 }, { type: Document, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [DOCUMENT]
                }] }], propDecorators: { maxWidth: [{
                type: HostBinding,
                args: ['style.max-width']
            }, {
                type: Input
            }], withoutBodyPadding: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }] } });

class BbDialogHeader {
    // Outputs.
    closeRequested = new EventEmitter();
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogHeader, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "18.2.8", type: BbDialogHeader, isStandalone: true, selector: "[bb-dialog-header]", outputs: { closeRequested: "closeRequested" }, host: { classAttribute: "bb-dialog-header" }, ngImport: i0, template: "<div class=\"bb-dialog-header-row\">\n    <div class=\"bb-dialog-header-content\">\n        <ng-content></ng-content>\n    </div>\n\n    @if (closeRequested?.observed) {\n        <button (click)=\"closeRequested?.emit()\"\n                type=\"button\"\n                class=\"bb-dialog-header-close\">\n        </button>\n    }\n</div>\n\n<nav class=\"bb-dialog-header-navigation\">\n    <ng-content select=\"[bb-dialog-link]\"></ng-content>\n</nav>\n", styles: [".bb-dialog-header{display:flex;min-height:3.75rem;flex-direction:column;justify-content:center;background-color:#fff;border-top-left-radius:.25rem;border-bottom:1px solid #d8d8d8;border-top-right-radius:.25rem}.bb-dialog-header .bb-dialog-header-content,.bb-dialog-header .bb-dialog-header-content h1{color:#111;max-width:100%;font-weight:500;overflow:hidden;font-size:1.25rem;white-space:nowrap;text-overflow:ellipsis}.bb-dialog-header-row,.bb-dialog-header-navigation{width:100%;display:flex;align-items:center}.bb-dialog-header-row{padding:.75rem 1.5rem}.bb-dialog-header-navigation{width:100%;overflow-x:auto;padding:0 1.5rem;white-space:nowrap}.bb-dialog-header-navigation:empty{margin:0;height:0;padding:0;visibility:hidden}.bb-dialog-header-content{flex:1}.bb-dialog-header-close{width:2rem;height:2rem;border:none;min-width:2rem;min-height:2rem;overflow:hidden;position:relative;margin-left:.75rem;display:inline-block;margin-right:-.75rem;border-radius:.25rem;background-color:transparent}.bb-dialog-header-close:before,.bb-dialog-header-close:after{top:50%;left:20%;width:60%;content:\"\";height:.125rem;margin-top:-1px;position:absolute;background-color:#999}.bb-dialog-header-close:before{transform:rotate(45deg)}.bb-dialog-header-close:after{transform:rotate(-45deg)}.bb-dialog-header-close:hover,.bb-dialog-header-close:focus{background-color:#e6e6e6}.bb-dialog-header-close:active{background-color:#d9d9d9}\n"], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogHeader, decorators: [{
            type: Component,
            args: [{ selector: '[bb-dialog-header]', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, host: { 'class': 'bb-dialog-header' }, preserveWhitespaces: false, standalone: true, template: "<div class=\"bb-dialog-header-row\">\n    <div class=\"bb-dialog-header-content\">\n        <ng-content></ng-content>\n    </div>\n\n    @if (closeRequested?.observed) {\n        <button (click)=\"closeRequested?.emit()\"\n                type=\"button\"\n                class=\"bb-dialog-header-close\">\n        </button>\n    }\n</div>\n\n<nav class=\"bb-dialog-header-navigation\">\n    <ng-content select=\"[bb-dialog-link]\"></ng-content>\n</nav>\n", styles: [".bb-dialog-header{display:flex;min-height:3.75rem;flex-direction:column;justify-content:center;background-color:#fff;border-top-left-radius:.25rem;border-bottom:1px solid #d8d8d8;border-top-right-radius:.25rem}.bb-dialog-header .bb-dialog-header-content,.bb-dialog-header .bb-dialog-header-content h1{color:#111;max-width:100%;font-weight:500;overflow:hidden;font-size:1.25rem;white-space:nowrap;text-overflow:ellipsis}.bb-dialog-header-row,.bb-dialog-header-navigation{width:100%;display:flex;align-items:center}.bb-dialog-header-row{padding:.75rem 1.5rem}.bb-dialog-header-navigation{width:100%;overflow-x:auto;padding:0 1.5rem;white-space:nowrap}.bb-dialog-header-navigation:empty{margin:0;height:0;padding:0;visibility:hidden}.bb-dialog-header-content{flex:1}.bb-dialog-header-close{width:2rem;height:2rem;border:none;min-width:2rem;min-height:2rem;overflow:hidden;position:relative;margin-left:.75rem;display:inline-block;margin-right:-.75rem;border-radius:.25rem;background-color:transparent}.bb-dialog-header-close:before,.bb-dialog-header-close:after{top:50%;left:20%;width:60%;content:\"\";height:.125rem;margin-top:-1px;position:absolute;background-color:#999}.bb-dialog-header-close:before{transform:rotate(45deg)}.bb-dialog-header-close:after{transform:rotate(-45deg)}.bb-dialog-header-close:hover,.bb-dialog-header-close:focus{background-color:#e6e6e6}.bb-dialog-header-close:active{background-color:#d9d9d9}\n"] }]
        }], propDecorators: { closeRequested: [{
                type: Output
            }] } });

class BbDialogLink {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogLink, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.8", type: BbDialogLink, isStandalone: true, selector: "[bb-dialog-link]", host: { classAttribute: "bb-dialog-link" }, ngImport: i0, template: "<ng-content></ng-content>\n", styles: [".bb-dialog-link{border:none;height:2rem;display:flex;font-size:1rem;appearance:none;font-weight:500;white-space:nowrap;align-items:center;color:#6b5c5c!important;background-color:transparent;text-decoration:none!important;border-bottom:2px solid transparent}.bb-dialog-link+.bb-dialog-link{margin-left:.75rem}.bb-dialog-link:hover:not(.disabled){border-bottom:2px solid rgba(117,119,148,.2)}.bb-dialog-link.active,.bb-dialog-link.active:hover{color:#5b53ff!important;border-bottom-color:#5b53ff}.bb-dialog-link.disabled,.bb-dialog-link.disabled:focus,.bb-dialog-link.disabled:hover,.bb-dialog-link.disabled:active{opacity:.4;cursor:default;box-shadow:none}\n"], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogLink, decorators: [{
            type: Component,
            args: [{ selector: '[bb-dialog-link]', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, host: { 'class': 'bb-dialog-link' }, preserveWhitespaces: false, standalone: true, template: "<ng-content></ng-content>\n", styles: [".bb-dialog-link{border:none;height:2rem;display:flex;font-size:1rem;appearance:none;font-weight:500;white-space:nowrap;align-items:center;color:#6b5c5c!important;background-color:transparent;text-decoration:none!important;border-bottom:2px solid transparent}.bb-dialog-link+.bb-dialog-link{margin-left:.75rem}.bb-dialog-link:hover:not(.disabled){border-bottom:2px solid rgba(117,119,148,.2)}.bb-dialog-link.active,.bb-dialog-link.active:hover{color:#5b53ff!important;border-bottom-color:#5b53ff}.bb-dialog-link.disabled,.bb-dialog-link.disabled:focus,.bb-dialog-link.disabled:hover,.bb-dialog-link.disabled:active{opacity:.4;cursor:default;box-shadow:none}\n"] }]
        }] });

class BbDialogActions {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogActions, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.8", type: BbDialogActions, isStandalone: true, selector: "[bb-dialog-actions]", host: { classAttribute: "bb-dialog-actions" }, ngImport: i0, template: "<bb-form-group class=\"end\">\n    <ng-content></ng-content>\n</bb-form-group>\n", styles: [".bb-dialog-actions{display:block;padding:1.25rem 1.5rem;background-color:#fff;border-top:1px solid #d8d8d8;border-bottom-left-radius:.25rem;border-bottom-right-radius:.25rem}\n"], dependencies: [{ kind: "component", type: BbFormGroup, selector: "bb-form-group", inputs: ["grouped"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogActions, decorators: [{
            type: Component,
            args: [{ selector: '[bb-dialog-actions]', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, host: { 'class': 'bb-dialog-actions' }, preserveWhitespaces: false, standalone: true, imports: [BbFormGroup], template: "<bb-form-group class=\"end\">\n    <ng-content></ng-content>\n</bb-form-group>\n", styles: [".bb-dialog-actions{display:block;padding:1.25rem 1.5rem;background-color:#fff;border-top:1px solid #d8d8d8;border-bottom-left-radius:.25rem;border-bottom-right-radius:.25rem}\n"] }]
        }] });

const bbDialogContainerAnimation = trigger('bbDialogContainerAnimation', [
    transition(':enter', [
        group([
            query('.bb-dialog-container-backdrop', [
                style({ opacity: 0 }),
                animate('180ms cubic-bezier(0, 0, 0.2, 1)', style({ opacity: 1 }))
            ]),
            query('.bb-dialog-container-component', [
                style({ opacity: 0, transform: 'scale(0.96)' }),
                animate('180ms cubic-bezier(0, 0, 0.2, 1)', style({ opacity: 1, transform: 'none' }))
            ])
        ])
    ]),
    transition(':leave', [
        group([
            query('.bb-dialog-container-backdrop', [
                animate('180ms cubic-bezier(0, 0, 0.2, 1)', style({ opacity: 0 }))
            ]),
            query('.bb-dialog-container-component', [
                animate('180ms cubic-bezier(0, 0, 0.2, 1)', style({
                    opacity: 0,
                    transform: 'scale(0.96)'
                }))
            ])
        ])
    ])
]);

class BbDialogInsertion {
    viewContainerRef;
    constructor(viewContainerRef) {
        this.viewContainerRef = viewContainerRef;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogInsertion, deps: [{ token: i0.ViewContainerRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.8", type: BbDialogInsertion, isStandalone: true, selector: "[bbDialogInsertion]", ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogInsertion, decorators: [{
            type: Directive,
            args: [{
                    selector: '[bbDialogInsertion]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: i0.ViewContainerRef }] });

class BbDialogContainer {
    // Dependencies.
    _config = inject(DIALOG_CONFIG, { optional: true });
    // Readonly data.
    disableContainerAnimations = this._config?.disableContainerAnimations ?? false;
    // Views.
    backdropElementRef;
    // Views.
    insertion;
    // Data.
    componentRef = null;
    childComponentType = null;
    // State.
    overlayClicked$ = new Subject();
    // Subscription.
    _subscription = new Subscription();
    get onOverlayClicked() {
        return this.overlayClicked$.pipe(take(1));
    }
    ngOnInit() {
        // Load the child component.
        this.loadChildComponent(this.childComponentType);
        // Listen for mouse click on the dialog container.
        this.listenForClicks();
    }
    ngOnDestroy() {
        this._subscription?.unsubscribe();
    }
    onAnimationDone(event) {
        // Check if the dialog is being destroyed.
        if (event?.toState !== 'void') {
            return;
        }
        // Destroy the ref if it exists.
        this.componentRef?.destroy();
    }
    listenForClicks() {
        const element = this.backdropElementRef?.nativeElement ?? null;
        if (!element) {
            return;
        }
        const mouseDown$ = fromEvent(element, 'mousedown');
        const touchStart$ = fromEvent(element, 'touchstart');
        const start$ = race([mouseDown$, touchStart$]);
        const mouseUp$ = fromEvent(element, 'mouseup');
        const touchEnd$ = fromEvent(element, 'touchend');
        const end$ = race([mouseUp$, touchEnd$]);
        const clicks$ = start$.pipe(switchMap(() => end$));
        const subscription = clicks$.subscribe(() => this.overlayClicked$.next());
        this._subscription.add(subscription);
    }
    loadChildComponent(componentType) {
        // Get the view container ref and clear it.
        const viewContainerRef = this.insertion.viewContainerRef;
        viewContainerRef.clear();
        // Create an instance of the component.
        this.componentRef = viewContainerRef.createComponent(componentType);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogContainer, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.8", type: BbDialogContainer, isStandalone: true, selector: "bb-dialog-container", host: { listeners: { "@bbDialogContainerAnimation.done": "onAnimationDone($event)" }, properties: { "@bbDialogContainerAnimation": "true", "@.disabled": "disableContainerAnimations" }, classAttribute: "bb-dialog-container" }, viewQueries: [{ propertyName: "backdropElementRef", first: true, predicate: ["backdrop"], descendants: true, static: true }, { propertyName: "insertion", first: true, predicate: BbDialogInsertion, descendants: true, static: true }], ngImport: i0, template: "<div #backdrop\n     class=\"bb-dialog-container-backdrop\"></div>\n\n<div class=\"bb-dialog-container-component\">\n    <ng-container bbDialogInsertion></ng-container>\n</div>\n", styles: [".bb-dialog-container,.bb-dialog-container-backdrop{inset:0;z-index:1000;position:fixed}.bb-dialog-container{display:flex}.bb-dialog-container-backdrop{will-change:opacity;background-color:#1119}.bb-dialog-container-component{width:100%;margin:auto;z-index:1001;display:block;padding:1.5rem;pointer-events:none;will-change:opacity,transform}\n"], dependencies: [{ kind: "directive", type: BbDialogInsertion, selector: "[bbDialogInsertion]" }], animations: [bbDialogContainerAnimation], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogContainer, decorators: [{
            type: Component,
            args: [{ selector: 'bb-dialog-container', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, host: {
                        'class': 'bb-dialog-container',
                        '[@bbDialogContainerAnimation]': 'true',
                        '[@.disabled]': 'disableContainerAnimations'
                    }, preserveWhitespaces: false, animations: [bbDialogContainerAnimation], standalone: true, imports: [BbDialogInsertion], template: "<div #backdrop\n     class=\"bb-dialog-container-backdrop\"></div>\n\n<div class=\"bb-dialog-container-component\">\n    <ng-container bbDialogInsertion></ng-container>\n</div>\n", styles: [".bb-dialog-container,.bb-dialog-container-backdrop{inset:0;z-index:1000;position:fixed}.bb-dialog-container{display:flex}.bb-dialog-container-backdrop{will-change:opacity;background-color:#1119}.bb-dialog-container-component{width:100%;margin:auto;z-index:1001;display:block;padding:1.5rem;pointer-events:none;will-change:opacity,transform}\n"] }]
        }], propDecorators: { backdropElementRef: [{
                type: ViewChild,
                args: ['backdrop', { static: true }]
            }], insertion: [{
                type: ViewChild,
                args: [BbDialogInsertion, { static: true }]
            }], onAnimationDone: [{
                type: HostListener,
                args: ['@bbDialogContainerAnimation.done', ['$event']]
            }] } });

let nextUniqueId = 0;
class BbDialogConfirm {
    // Dependencies.
    _config = inject(BbDialogConfig);
    _dialogRef = inject(BbDialogRef);
    // Readonly data.
    dialogId = `bb-dialog-confirm-${nextUniqueId++}`;
    title = this._config?.data?.title ?? null;
    description = this._config?.data?.description ?? null;
    confirmButtonText = this._config?.data?.confirmButtonText ?? 'dialog.default_confirm_button';
    cancelButtonText = this._config?.data?.cancelButtonText ?? 'dialog.default_cancel_button';
    confirmButtonClass = this._config?.data?.confirmButtonClass ?? null;
    cancelButtonClass = this._config?.data?.cancelButtonClass ?? null;
    width = this._config?.data?.width ?? '30rem';
    // Views.
    confirmButton;
    ngAfterViewInit() {
        // Initial focus the button.
        this.confirmButton?.focus();
    }
    onClose(result) {
        this._dialogRef.close(result);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogConfirm, deps: [], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "18.2.8", type: BbDialogConfirm, isStandalone: true, selector: "bb-dialog-confirm", host: { classAttribute: "bb-dialog-confirm" }, viewQueries: [{ propertyName: "confirmButton", first: true, predicate: ["confirm"], descendants: true, read: BbButton }], ngImport: i0, template: "<div [attr.aria-labelledby]=\"dialogId\"\n     [maxWidth]=\"width\"\n     bb-dialog-modal\n     bbFocusTrap>\n    <header bb-dialog-header>\n        @if (title; as titleContent) {\n            <h1 [id]=\"dialogId\">\n                <ng-template [bbTemplate]=\"titleContent\">\n                    {{ $any(titleContent) | bbLocalize:{optional: true} }}\n                </ng-template>\n            </h1>\n        }\n    </header>\n\n    @if (description; as descriptionContent) {\n        <p class=\"bb-dialog-confirm-description\">\n            <ng-template [bbTemplate]=\"descriptionContent\">\n                {{ $any(descriptionContent) | bbLocalize:{optional: true} }}\n            </ng-template>\n        </p>\n    }\n\n    <footer class=\"bb-dialog-confirm-footer\">\n        @if (cancelButtonText; as cancelButtonTextLabel) {\n            <button [ngClass]=\"cancelButtonClass ? cancelButtonClass : 'secondary full'\"\n                    (click)=\"onClose(false)\"\n                    bb-button\n                    type=\"button\">\n                {{ cancelButtonTextLabel | bbLocalize:{optional: true} }}\n            </button>\n        }\n        @if (confirmButtonText; as confirmButtonTextLabel) {\n            <button #confirm\n                    [ngClass]=\"confirmButtonClass ? confirmButtonClass : 'destructive full'\"\n                    (click)=\"onClose(true)\"\n                    bb-button\n                    type=\"button\">\n                {{ confirmButtonTextLabel | bbLocalize:{optional: true} }}\n            </button>\n        }\n    </footer>\n</div>\n", styles: [".bb-dialog-confirm{display:block}.bb-dialog-confirm>.bb-dialog-modal>.bb-dialog-modal-body{background-color:#fff;border-bottom-left-radius:.25rem;border-bottom-right-radius:.25rem}.bb-dialog-confirm-description{line-height:2;font-size:.875rem}.bb-dialog-confirm-footer{display:flex;margin-top:1.5rem;flex-direction:column}.bb-dialog-confirm-footer>*+*{margin-top:1rem}@media only screen and (min-width: 768px){.bb-dialog-confirm-footer{flex-direction:row}.bb-dialog-confirm-footer>*+*{margin-top:0;margin-left:1rem}}\n"], dependencies: [{ kind: "component", type: BbDialogModal, selector: "[bb-dialog-modal]", inputs: ["maxWidth", "withoutBodyPadding"] }, { kind: "component", type: BbDialogHeader, selector: "[bb-dialog-header]", outputs: ["closeRequested"] }, { kind: "directive", type: BbFocusTrap, selector: "[bbFocusTrap]" }, { kind: "pipe", type: BbLocalize, name: "bbLocalize" }, { kind: "directive", type: BbTemplate, selector: "[bbTemplate]", inputs: ["bbTemplate"] }, { kind: "directive", type: NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "component", type: BbButton, selector: "button[bb-button]", inputs: ["disabled", "loading"], exportAs: ["bbButton"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbDialogConfirm, decorators: [{
            type: Component,
            args: [{ selector: 'bb-dialog-confirm', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, preserveWhitespaces: false, host: { 'class': 'bb-dialog-confirm' }, standalone: true, imports: [BbDialogModal, BbDialogHeader, BbFocusTrap, BbLocalize, BbTemplate, NgClass, BbButton], template: "<div [attr.aria-labelledby]=\"dialogId\"\n     [maxWidth]=\"width\"\n     bb-dialog-modal\n     bbFocusTrap>\n    <header bb-dialog-header>\n        @if (title; as titleContent) {\n            <h1 [id]=\"dialogId\">\n                <ng-template [bbTemplate]=\"titleContent\">\n                    {{ $any(titleContent) | bbLocalize:{optional: true} }}\n                </ng-template>\n            </h1>\n        }\n    </header>\n\n    @if (description; as descriptionContent) {\n        <p class=\"bb-dialog-confirm-description\">\n            <ng-template [bbTemplate]=\"descriptionContent\">\n                {{ $any(descriptionContent) | bbLocalize:{optional: true} }}\n            </ng-template>\n        </p>\n    }\n\n    <footer class=\"bb-dialog-confirm-footer\">\n        @if (cancelButtonText; as cancelButtonTextLabel) {\n            <button [ngClass]=\"cancelButtonClass ? cancelButtonClass : 'secondary full'\"\n                    (click)=\"onClose(false)\"\n                    bb-button\n                    type=\"button\">\n                {{ cancelButtonTextLabel | bbLocalize:{optional: true} }}\n            </button>\n        }\n        @if (confirmButtonText; as confirmButtonTextLabel) {\n            <button #confirm\n                    [ngClass]=\"confirmButtonClass ? confirmButtonClass : 'destructive full'\"\n                    (click)=\"onClose(true)\"\n                    bb-button\n                    type=\"button\">\n                {{ confirmButtonTextLabel | bbLocalize:{optional: true} }}\n            </button>\n        }\n    </footer>\n</div>\n", styles: [".bb-dialog-confirm{display:block}.bb-dialog-confirm>.bb-dialog-modal>.bb-dialog-modal-body{background-color:#fff;border-bottom-left-radius:.25rem;border-bottom-right-radius:.25rem}.bb-dialog-confirm-description{line-height:2;font-size:.875rem}.bb-dialog-confirm-footer{display:flex;margin-top:1.5rem;flex-direction:column}.bb-dialog-confirm-footer>*+*{margin-top:1rem}@media only screen and (min-width: 768px){.bb-dialog-confirm-footer{flex-direction:row}.bb-dialog-confirm-footer>*+*{margin-top:0;margin-left:1rem}}\n"] }]
        }], propDecorators: { confirmButton: [{
                type: ViewChild,
                args: ['confirm', { read: BbButton, static: false }]
            }] } });

class Dialog {
    _applicationRef;
    _environmentInjector;
    constructor(_applicationRef, _environmentInjector) {
        this._applicationRef = _applicationRef;
        this._environmentInjector = _environmentInjector;
    }
    open(componentType, config) {
        return this.createDialog(componentType, config);
    }
    async confirm(title, description, options) {
        const dialogRef = this.open(BbDialogConfirm, {
            data: { title, description, ...(options ?? {}) }
        });
        return await firstValueFrom(dialogRef.afterClosed);
    }
    createDialog(componentType, config) {
        // Create a reference.
        const dialogRef = new BbDialogRef();
        const environmentInjector = createEnvironmentInjector([
            { provide: BbDialogConfig, useValue: config ?? {} },
            { provide: BbDialogRef, useValue: dialogRef }
        ], this._environmentInjector);
        const dialogComponentRef = createComponent(BbDialogContainer, { environmentInjector });
        // Attach the host view to the application.
        this._applicationRef.attachView(dialogComponentRef.hostView);
        try {
            // Remove the current focus.
            const activeElement = document.activeElement;
            activeElement.blur();
            // Append the element to the DOM.
            document.body.appendChild(dialogComponentRef.location.nativeElement);
        }
        catch {
            // Don't do anything, because it must've failed.
        }
        // Set the child component type on the component.
        dialogComponentRef.instance.childComponentType = componentType;
        dialogComponentRef.instance.onOverlayClicked.subscribe(() => dialogRef.onOverlayClicked());
        // Listen for close events.
        dialogRef.afterClosed.subscribe(() => this.deleteDialog(dialogComponentRef));
        // Return the reference.
        return dialogRef;
    }
    deleteDialog(dialogComponentRef) {
        // Only continue if the dialog component ref exists.
        if (!dialogComponentRef) {
            return;
        }
        // Detach the dialog.
        this._applicationRef.detachView(dialogComponentRef.hostView);
        // Destroy the dialog.
        dialogComponentRef.destroy();
        dialogComponentRef = null;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: Dialog, deps: [{ token: i0.ApplicationRef }, { token: i0.EnvironmentInjector }], target: i0.ɵɵFactoryTarget.Injectable });
    static ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: Dialog, providedIn: 'root' });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: Dialog, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: i0.ApplicationRef }, { type: i0.EnvironmentInjector }] });

class BbConfirm {
    // Dependencies.
    _dialog = inject(Dialog);
    // Inputs.
    bbConfirmTitle = 'dialog.default_title';
    bbConfirmDescription = 'dialog.default_description';
    bbConfirmButtonText = null;
    bbCancelButtonText = null;
    // Outputs.
    confirm = new EventEmitter();
    async onClick(event) {
        const confirmed = await this._dialog.confirm(this.bbConfirmTitle, this.bbConfirmDescription, {
            confirmButtonText: this.bbConfirmButtonText,
            cancelButtonText: this.bbCancelButtonText
        });
        if (!confirmed) {
            return;
        }
        this.confirm.emit(event);
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbConfirm, deps: [], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.2.8", type: BbConfirm, isStandalone: true, selector: "[bbConfirm]", inputs: { bbConfirmTitle: "bbConfirmTitle", bbConfirmDescription: "bbConfirmDescription", bbConfirmButtonText: "bbConfirmButtonText", bbCancelButtonText: "bbCancelButtonText" }, outputs: { confirm: "bbConfirm" }, host: { listeners: { "click": "onClick($event)" } }, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: BbConfirm, decorators: [{
            type: Directive,
            args: [{
                    selector: '[bbConfirm]',
                    standalone: true
                }]
        }], propDecorators: { bbConfirmTitle: [{
                type: Input
            }], bbConfirmDescription: [{
                type: Input
            }], bbConfirmButtonText: [{
                type: Input
            }], bbCancelButtonText: [{
                type: Input
            }], confirm: [{
                type: Output,
                args: ['bbConfirm']
            }], onClick: [{
                type: HostListener,
                args: ['click', ['$event']]
            }] } });

function provideDialogConfig(config) {
    return makeEnvironmentProviders([
        { provide: DIALOG_CONFIG, useValue: config ?? {} }
    ]);
}

class DialogModule {
    static forRoot(config) {
        return {
            ngModule: DialogModule,
            providers: [
                provideDialogConfig(config)
            ]
        };
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: DialogModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
    static ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "18.2.8", ngImport: i0, type: DialogModule, imports: [BbDialogContainer,
            BbDialogOverlay,
            BbDialogModal,
            BbDialogHeader,
            BbDialogLink,
            BbDialogActions,
            BbDialogConfirm,
            BbDialogInsertion,
            BbConfirm], exports: [BbDialogOverlay,
            BbDialogModal,
            BbDialogHeader,
            BbDialogLink,
            BbDialogActions,
            BbConfirm] });
    static ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: DialogModule, imports: [BbDialogActions,
            BbDialogConfirm] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.8", ngImport: i0, type: DialogModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [
                        BbDialogContainer,
                        BbDialogOverlay,
                        BbDialogModal,
                        BbDialogHeader,
                        BbDialogLink,
                        BbDialogActions,
                        BbDialogConfirm,
                        BbDialogInsertion,
                        BbConfirm
                    ],
                    exports: [
                        BbDialogOverlay,
                        BbDialogModal,
                        BbDialogHeader,
                        BbDialogLink,
                        BbDialogActions,
                        BbConfirm
                    ]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { BbConfirm, BbDialogActions, BbDialogConfig, BbDialogHeader, BbDialogLink, BbDialogModal, BbDialogOverlay, BbDialogRef, DIALOG_CONFIG, Dialog, DialogConfig, DialogModule, provideDialogConfig };
//# sourceMappingURL=bravobit-bb-foundation-dialog.mjs.map
